"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__version__ = 'v2.5.3'

import argparse
import importlib
import os
import shutil
import sys
import shlex
import json
import concurrent.futures

from maryam.core import basedir
import multiprocessing as mp

class turn_off:
	def __enter__(self):
		self._original_stdout = sys.stdout
		sys.stdout = open(os.devnull, 'w')

	def __exit__(self, exc_type, exc_val, exc_tb):
		sys.stdout.close()
		sys.stdout = self._original_stdout

from .core import core

class initialize(core):

	def __init__(self, mode):
		core.__init__(self)
		self._mode = mode
		self._config = {
				'name': 'maryam',
				'module_ext': '.py',
				'data_directory_name': 'data',
				'module_directory_name': 'modules',
				'workspaces_directory_name': '.maryam/workspaces'
			}
		self._name = self._config['name']
		self._prompt_template = '%s[%s] > '
		self._base_prompt = self._prompt_template % ('', self._name)
		self.path = basedir.BASEDIR
		self.data_path = os.path.join(
			self.path, self._config['data_directory_name'])
		self.core_path = os.path.join(
			self.path, 'core')
		self.module_path = os.path.join(
			self.path, self._config['module_directory_name'])
		self.module_ext = self._config[
			'module_ext']
		self.module_dirname = self._config[
			'module_directory_name']
		self.workspaces_dirname = self._config['workspaces_directory_name']
		self._init_framework_options()
		self._init_home()
		self._init_workspace('default')
		self._load_modules()
		if mode != 'execute':
			self.__version__ = __version__
			self._check_version()
			self.show_banner()

	def _init_framework_options(self):
		self._global_options_['proxy'] = ('proxy', None, False,
							 'proxy server (address:port)')
		self._global_options_['agent'] = (
			'agent', 'Mozilla/5.0 (X11; Linux x86_64; rv:75.0) Gecko/20100101 Firefox/75.0',\
			 True, 'user-agent')
		self._global_options_['rand_agent'] = ('rand_agent', False, True, 'setting random user-agent')
		self._global_options_['timeout'] = ('timeout', 10, True, 'socket timeout (seconds)')
		self._global_options_['verbosity'] = ('verbosity', 1, True, \
			'verbosity level (0 = minimal, 1 = verbose, 2 = debug)')
		self._global_options_['update_check'] = ('update_check', True, True,\
			'checking the framework version before running')
		self._global_options_['protocol'] = ('protocol', 'https', True,\
			'default protocol')
		self._global_options_['api_mode'] = ('api_mode', False, True,\
			'Activating API mode causes results will be shown in JSON format(warn: it doesnt show any other information)')
		for key in self._global_options_:
			self._global_options[key] = self._global_options_[key][1]

	def _init_home(self):
		self._home = os.path.expanduser('~')
		# initialize home folder
		if not os.path.exists(self._home):
			os.makedirs(self._home)

	def _load_a_module(self, mod_name, mod_path):
		spec = importlib.util.spec_from_file_location(mod_name, mod_path)
		module = importlib.util.module_from_spec(spec)
		spec.loader.exec_module(module)
		return module

	def _init_util_classes(self, require='*'):
		util_path = os.path.join(self.core_path, 'util')
		for dirpath, _, files in os.walk(util_path, followlinks=True):
			if not dirpath.endswith(require) and require != '*' and util_path != dirpath: continue
			for file in filter(lambda f: f.endswith('.py')\
						and not f.startswith('__'), files):
				mod_name = file.split('.')[0]
				mod_path = os.path.join(dirpath, file)
				try:
					module = self._load_a_module(mod_name, mod_path)
				except Exception as e:
					self.error(f"Util class '{mod_name}' has been disabled due to this error: {e}", 'initial', '_init_util_classes')
				else:
					exec(f"self.{mod_name} = module.main")
					exec(f"self.{mod_name}.framework = self")

	def _load_modules(self, require='*'):
		self.loaded_category = {}
		self._loaded_modules = core._loaded_modules
		self._init_util_classes(require)
		for dirpath, dirnames, _ in os.walk(self.module_path, followlinks=True):
			# Each section
			for section in filter(lambda d: not d.startswith('__'), dirnames):
				category = section
				if require != category and require != '*': continue
				self._cat_module_names[category] = []
				section = os.path.join(dirpath, section)
				for _, _, files in os.walk(section):
					# Each file
					for file in filter(lambda f: f.endswith(self.module_ext) and not f.startswith('__'), files):
						mod_path = os.path.join(section, file)
						mod_load_name = f"__{file.split('.')[0]}"
						mod_disp_name = file.split('.')[0]
						try:
							module = self._load_a_module(mod_load_name, mod_path)
						except Exception as e:
							if not self._global_options['api_mode']:
								self.error(f"Module name '{mod_disp_name}' has been disabled due to this error: {e}", 'initial', '_load_modules')
						else:
							self._loaded_modules[mod_disp_name] = module
							self._cat_module_names[category].append(mod_disp_name)
							self._module_names.append(mod_disp_name)

	# ////////////////////////
	#          API 		    //
	# ////////////////////////

	def do_web(self, params):
		'''Manage web/api interface'''
		if not params:
			self.help_web()
			return
		params = params.lower().split(' ')
		if 'api' not in sys.modules:
			from .web import api
		if params[0] == 'api':
			api_mode = self._global_options['api_mode']
			_mode = self._mode
			self._mode = 'api'
			if not api_mode:
				self._global_options['api_mode'] = True
			if len(params) == 3:
				host = params[1]
				port = params[2]
			else:
				host = '127.0.0.1'
				port = 1313

			api.run_app(self, host, port)
			self._global_options['api_mode'] = api_mode
			self._mode = _mode

	def help_web(self):
		print(getattr(self, 'do_web').__doc__)
		print('\tweb api => running api on 127.0.0.1:1313')
		print('\tweb api <HOST> <PORT> => running api')

	# ////////////////////////////////
	#           WORKSPACE 		    //
	# ////////////////////////////////

	def _init_workspace(self, workspace):
		if not workspace:
			return
		workspace = os.path.join(self._home, self.workspaces_dirname, workspace)
		if not os.path.exists(workspace):
			os.makedirs(workspace)
		self.workspace = core.workspace = workspace
		self.prompt = self._prompt_template % (self._base_prompt[:-3], self.workspace.split('/')[-1])
		self._load_config()
		return True

	def remove_workspaces(self, workspace):
		path = os.path.join(self._home, self.workspaces_dirname, workspace)
		try:
			shutil.rmtree(path)
		except OSError:
			return False
		if workspace == self.workspace.split('/')[-1]:
			self._init_workspace('default')
		return True

	def _get_workspaces(self):
		dirnames = []
		path = os.path.join(self._home, self.workspaces_dirname)
		dirnames = [name for name in os.listdir(path) if os.path.isdir(os.path.join(path, name))]
		return dirnames

	def show_banner(self):
		print(f"OWASP Maryam({__version__}): Open-source Intelligence Framework.\
			\nTo show the framework help, run 'help' command.")

	def show_workspaces(self):
		self.do_workspaces('list')

	def help_workspaces(self):
		print(getattr(self, 'do_workspaces').__doc__)
		print(f'{os.linesep}Usage: workspaces [add|select|delete|list]{os.linesep}')

	def do_reload(self, params=None):
		'''Reloads all modules'''
		self.output(f"Reloading...")
		if params:
			params = params.split()
			section = params[0]
		else:
			section = '*'
		self._load_modules(section)

	def do_workspaces(self, params):
		'''Manages workspaces'''
		if not params:
			self.help_workspaces()
			return
		params = params.split()
		arg = params.pop(0).lower()
		if arg == 'list':
			header = '\nWorkspaces\n'
			print(f"{header}{self.ruler * len(header[2:])}")
			for i in self._get_workspaces():
				print(''.ljust(5) + i)
			print('')
		elif arg in ['add', 'select']:
			if len(params) == 1:
				if not self._init_workspace(params[0]):
					self.output(f"Unable to initialize '{params[0]}' workspace.")
			else:
				print('Usage: workspace [add|select|delete] <name>')
		elif arg == 'delete':
			if len(params) == 1:
				if not self.remove_workspaces(params[0]):
					self.output(f"Unable to delete '{params[0]}' workspace.")
			else:
				print('Usage: workspace delete <name>')
		else:
			self.help_workspaces()

	# ////////////////////////////////
	#           MODULES 			//
	# ////////////////////////////////

	def alert_results(self, output, prefix='\t', depth=0, color='N'):
		if output == [] or output == {}:
			if depth != 0:
				self.alert_results('Without result', prefix='\t', depth=depth, color='R')
		elif isinstance(output, dict):
			for key, value in output.items():
				key = key.replace('_', ' ')
				if isinstance(value, list) or isinstance(value, dict):
					self.alert(f"{prefix*depth}{key.upper()}")
					self.alert_results(value, prefix=prefix, depth=depth + 1, color='G')
				else :
					value = value.strip().replace('\n', ' ').replace('\\x', ' ') if isinstance(value, str) else value
					self.alert(f"{prefix*depth}{key.upper()}")
					self.output(f"{prefix*(depth+1)}{value}", color)
			print('')
		elif isinstance(output, list):
			for key in output:
				self.alert_results(key, prefix='\t', depth=depth, color='G')
		else:
			output = output.strip().replace('\n', ' ').replace('\\x', ' ')
			self.output(f"{prefix*depth}{output}", color)

	def search_engine_results(self, output):
		for i in output['results']:
			self.output(i['t'])
			self.output(i['a'])
			self.output(i['c'])
			self.output(i['d'])
			print()

	def thread(self, *args):
		""" self, function, thread_count, engines, {...all args}, sources"""
		with concurrent.futures.ThreadPoolExecutor(max_workers=args[1]) as executor:
			[executor.submit(args[0], self, name, *args[3:-1]) for name in args[2] if name in args[-1]]

	def opt_proc(self, tool_name, args=None, output=None):
		""" processing the module name and its options to run"""
		mod = self._loaded_modules[tool_name]
		meta = mod.meta
		opts = meta['options']
		self.options = {}
		# Add default options
		opts += (('output', False, False, 'Save the output to the workspace', '--output', 'store_true', bool),)
		opts += (('api', False, False, 'Show results in the JSON format', '--api', 'store_true', bool),)
		opts += (('format', False, False, 'Beautifying JSON output if --api is used', '--format', 'store_true', bool),)
		description = f"{tool_name} {meta['version']}({meta['author']}) - \tdescription: {meta['description']}\n"
		parser = argparse.ArgumentParser(prog=tool_name, description=description)
		for option in opts:
			try:
				name, val, req, desc, op, act, typ = option
			except ValueError as e:
				self.error(f"{tool_name.title()}CodeError: options is too short. need more than {len(option)} option")
				return False
			name = f"--{name}" if not name.startswith('-') else name
			try:
				if act == 'store':
					parser.add_argument(op, name, help=desc, default=val, action='store', type=typ, required=req)
				else:
					parser.add_argument(op, name, help=desc, default=val, action=act, required=req)
			except argparse.ArgumentError as e:
				self.output(f"ModuleException: {str(e)}")
		# Initialize help menu
		format_help = parser.format_help()
		# Add comments
		if 'comments' in meta:
			format_help += 'Comments:'
			for comment in meta['comments']:
				prefix = '* '
				if comment.startswith('\t'):
					prefix = self.spacer + '- '
					comment = comment[1:]
				format_help += self.textwrapping(self.spacer, comment, 100)
			format_help += '\n'
		if 'sources' in meta:
			format_help += '\nSources:\n\t' + '\n\t'.join(meta['sources'])
		if 'examples' in meta:
			format_help += '\nExamples:\n\t' + '\n\t'.join(meta['examples'])
		if 'contributors' in meta:
			format_help += f"\nContributors:\n\t{meta['contributors']}"
		if 'required' in meta:
			format_help += '\nRequirements:\n\t' + '\n\t'.join(meta['required'])
		# If args is nothing
		if not args:
			print(format_help)
			return False
		else:
			# Initialite args
			if self._mode == 'execute':
				argv = sys.argv[3:]
				argx = parser.parse_args(argv)
			else:
				lexer = shlex.split(args)
				argx = parser.parse_args(lexer)
			# Set options
			self.options = vars(argx)
			try:
				if self.options['api'] or self._global_options['api_mode']:
					# Turn off framework prints till the executing of module
					with turn_off():
						results = mod.module_api(self)
						results['running_errors'] = self._error_stack
					if output == 'web_api':
						return results
					elif self.options['format']:
						print(json.dumps(results, indent=4))
					else:
						print(results)
					self._reset_error_stack()
				else:
					mod.module_run(self)
			except KeyboardInterrupt:
				pass
			except ImportError as e:
				self.output(f"{tool_name} requires some modules that can't be imported.")
				self.output(e)
				if 'required' in meta:
					self.output('Installing...')
					required = meta['required']
					with turn_off():
						section = []
						reqs = []
						for i in required:
							reqs.append(i)
						if reqs:
							self.output('required:')
							for i in reqs:
								self.output(i)
							error = self._dev_install_requirements(reqs)
							if error:
								self.output('Failed.')
								return
							else:
								self.output('Done.')
			except Exception as e:
				self.print_exception(where=tool_name, which_func='opt_proc')

	def mod_api_run(self, module):
		mod = self._loaded_modules[module]
		try:
			results = mod.module_api(self)
			results['running_errors'] = self._error_stack
			self._reset_error_stack()
			return results
		except KeyboardInterrupt:
			pass
		except Exception as e:
			self.print_exception()
		return False

	def run_tool(self, func, tool_name, args, output=None):
		try:
			if sys.platform == 'darwin' and mp.get_start_method() != 'fork':
				mp.set_start_method('fork', force=True)

			proc = mp.Process(target=getattr(self, func), args=(tool_name, args, output))
			proc.start()
			proc.join()
			if proc.is_alive():
				proc.kill()
			return
		except KeyboardInterrupt as e:
			print(f"\nStopping {tool_name} module...")
			if proc.is_alive():
				proc.kill()
		except Exception as e:
			self.print_exception()



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re


meta = {
	'name': 'Quora Search',
	'author': 'Aman Rawat',
	'version': '0.2',
	'description': 'Search your query in the quora.com and show the results.',
	'sources': ('google', 'bing', 'millionshort', 'etools', 'duckduckgo'),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
		('limit', 1, False, 'Search limit(number of pages, default=1)', '-l', 'store', int),
		('count', 50, False, 'Number of results per page(min=10, max=100, default=50)', '-c', 'store', int),
		('thread', 2, False, 'The number of engine that run per round(default=2)', '-t', 'store', int),
		('engine', 'bing', False, 'Engine names for search(default=bing)', '-e', 'store', str),
	),
	'examples': ('quora -q <QUERY> -l 15 --output',)
}

LINKS = []

def search(self, name, q, q_formats, limit, count):
	global LINKS
	engine = getattr(self, name)
	q = q_formats[f"{name}_q"] if f"{name}_q" in q_formats else q_formats['default_q']
	varnames = engine.__init__.__code__.co_varnames
	if 'limit' in varnames and 'count' in varnames:
		attr = engine(q, limit, count)
	elif 'limit' in varnames:
		attr = engine(q, limit)
	else:
		attr = engine(q)

	attr.run_crawl()
	if name == 'google':
		LINKS += [(x['a'], x['t']) for x in attr.results_original]
	LINKS += attr.links

def module_api(self):
	query = self.options['query']
	limit = self.options['limit']
	count = self.options['count']
	engines = self.options['engine'].split(',')
	output = {'links': [], 'usernames': []}
	q_formats = {
		'default_q': f"site:www.quora.com {query}",
		'millionshort_q': f'site:www.quora.com "{query}"',
		'qwant_q': f'site:www.quora.com {query}'
	}

	self.thread(search, self.options['thread'], engines, query, q_formats, limit, count, meta['sources'])

	links = list(set(LINKS))
	for link in self.reglib().filter(r"https?://(www\.)?quora\.com/profile/", links):
		if link not in output['usernames']:
			output['usernames'].append(link)

	for link in links:
		if re.search(r"https?://(www\.)?quora\.com", link) and '/profile' not in link:
			title = re.sub(r"https?://(www\.)?quora\.com/", '', link)
			title = title.replace('-', ' ')
			title = self.urlib(title).unquote.split('?')[0]
			output['links'].append([link, title])

	self.save_gather(output, 'search/quora', query, output=self.options.get('output'))
	return output

def module_run(self):
	output = module_api(self)
	self.alert('usernames')
	for user in output['usernames']:
		self.output(f"\t{user}", 'G')
	self.alert('links')
	for item in output['links']:
		title, link = item[1], item[0]
		self.output(title)
		self.output(f"\t{link}", 'G')
		print('')



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
import os
from collections import Counter
BASEDIR = os.path.abspath(os.path.join(os.path.dirname(os.path.realpath(__file__)), '../'))

class main:

	def __init__(self, docs: 'documents', form: 'documet form. e.g html', without_punc=True):
		""" show histogram plot of web term frequency """
		self.framework = main.framework
		self.form = form.lower()
		self.docs = docs.lower()
		self.without_punc = without_punc
		if self.form == 'html':
			pp = self.framework.page_parse(self.docs)
			pp.remove_html_tags
			self.docs = pp.page
		self.words = self.docs.split()
		if self.without_punc:
			self._punc()

	def remove_stopwords(self, rest):
		stops = open(os.path.join(BASEDIR, '../../', 'data', 'stopwords.csv')).read().split(',')
		self.words = [x for x in self.words if x not in stops and x not in rest]

	def _punc(self):
		self.words = re.findall(r"[\w\-_#]{2,}", self.docs)

	def _counter(self, last):
		""" last: number of terms to show in plot """
		bow = Counter(self.words)
		return bow.most_common(last)

	def plot_histogram(self, title, last, should_show=False):
		import pandas as pd
		import matplotlib.pyplot as plt

		most_common = self._counter(last)
		clean_tweets_no_urls = pd.DataFrame(most_common,
						columns=['words', 'count'])
		fig, ax = plt.subplots(figsize=(25, 25))
		clean_tweets_no_urls.sort_values(by='count').plot.barh(x='words',
									y='count',
									ax=ax,
									color="black")
		ax.set_title(title)

		filename = os.path.join(self.framework.workspace,title.replace(' ','_')+'.png')
		plt.savefig(filename, format="png")

		if should_show:
			plt.show()

		return filename



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

meta = {
	'name': 'Phone Number Search',
	'author': 'Kaushik',
	'version': '0.1',
	'description': 'A Search Engine for Phone Number Enumeration.',
	'sources': ('numverify'),
	'options': (
			('number', None, True, 'Phone number (include area code)', '-n', 'store', str),
	),
	'examples': ('phone_number_search -n +911234567890')
}

def module_api(self):
	num = self.options['number']
	run = self.numverify(num)
	run.run_crawl()
	output = run.json
	self.save_gather(output, 'osint/phone_number_search', num, output=self.options['output'])
	return output

def module_run(self):
	output = module_api(self)
	if output['valid']:
		self.alert_results(output)
	else:
		self.error('Invalid Number!', 'phone_number_search', 'module_run')
		self.error("Number must be of the form '+{area code}{ten digit number}'", 'phone_number_search', 'module_run')



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from hashlib import sha1
import re

class main:

	def __init__(self, q, limit=4):
		""" netcraft.com for find dns

			q 		  : Query for search
		"""
		self.framework = main.framework
		self.q = q
		self.limit = limit
		self.base_url = f"https://searchdns.netcraft.com/?restriction=site+ends+with&host={q}"
		self._pages = ''

	def request(self, url, cookies=None):
		cookies = cookies or {}
		try:
			req = self.framework.request(url=url, cookies=cookies)
		except Exception as e:
			self.framework.error(f"ConnectionError {e}.", 'util/netcraft', 'request')
			self.framework.error('Netcraft is missed!', 'util/netcraft', 'request')
			return False
		return req

	def get_next(self, resp):
		link_regx = re.compile(r'<a class="btn-info" href="(.*?)">Next Page <i')
		link = link_regx.findall(resp)
		if not link:
			return False
		link = link[0]

		url = 'https://searchdns.netcraft.com' + link.replace(' ', '%20')
		return url

	def get_cookies(self, headers):
		if 'set-cookie' in headers:
			cookie = headers['set-cookie']
			cookies = {}
			cookies_list = cookie[0:cookie.find(';')].split('=')
			cookies[cookies_list[0]] = cookies_list[1]
			cookies['netcraft_js_verification_response'] = sha1(
				self.framework.urlib(cookies_list[1]).unquote.encode('utf-8')).hexdigest()
		else:
			cookies = {}
		return cookies

	def run_crawl(self):
		start_url = self.base_url
		resp = self.request(start_url)
		if not resp:
			return
		count = 1
		cookies = self.get_cookies(resp.headers)

		while True:
			self.framework.verbose(f"[NETCRAFT] Searching in {count} page...")
			resp = self.request(self.base_url, cookies).text
			if not resp:
				return
			self._pages += resp
			if count+1 == self.limit:
				break
			next_link = self.get_next(resp)
			if not next_link:
				return
			self.base_url = next_link
			count += 1

	@property
	def pages(self):
		return self._pages

	@property
	def dns(self):
		dns = self.framework.page_parse(str(self._pages)).get_dns(self.q)
		return dns



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

meta = {
	'name': 'Image Search',
	'author': 'Saeed',
	'version': '0.1',
	'description': 'Search in open-sources to find images.',
	'sources': ('bing', 'google'),
	'options': (
		('query', None, True, 'Query, host Name, company Name, keyword, , etc', '-q', 'store', str),
		('engines', 'google', True, 'Search engines with comma separator', '-e', 'store', str),
		('thread', 3, False, 'The number of engine that run per round(default=2)', '-t', 'store', int),
	),
	'examples': ('image_search -q amazon -e google,bing,qwant',)
}
	
IMGS = []
def remove_dups(self):
	urls = []
	new = []
	for i in IMGS:
		a = self.urlib(i['i'].lower()).sub_service()
		if a not in urls:
			urls.append(a)
			new.append(i)
	return new

def search(self, name, q):
	global IMGS
	try:
		engine = getattr(self, name + '_images')
		q = q
		varnames = engine.__init__.__code__.co_varnames
		attr = engine(q)

		attr.run_crawl()
		IMGS += attr.results
	except Exception as e:
		print(e)

def module_api(self):
	query = self.options['query']
	engines = self.options['engines'].lower().split(',')

	self.thread(search, self.options['thread'], engines, query, meta['sources'])
	INGS = remove_dups(self)
	output = {'results' : IMGS}
	self.save_gather(output, 'osint/image_search', query, output=self.options['output'])
	return output

def module_run(self):
	for i in module_api(self)['results']:
		self.output(i['a'])
		self.output(i['i'])
		self.output(i['t'])
		self.output(i['d'])
		print()



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from requests.exceptions import Timeout
import datetime

class main:

	def __init__(self, q, limit=20, verbose=False, daterange=-1):
		""" tweet search engine

				q         : query for search
				limit     : maximum result count
				verbose   : print entire json 
				daterange : time range to fetch tweets for
		"""
		self.framework = main.framework
		self.q = q
		self.max = limit
		self._json = {}
		self._tweets = {}
		self._verbose = verbose
		self._daterange = daterange

	def run_crawl(self):
		self.framework.verbose('[TWEETSEARCH] Searching the twitter domain...')
		self.header = {
			'authorization': ('Bearer AAAAAAAAAAAAAAAAAAAAANRILgAAAAAAnNw'
					'IzUejRCOuH5E6I8xnZz4puTs%3D1Zv7ttfk8LF81IUq1'
					'6cHjhLTvJu4FA33AGWWjCpTnA'),
			'x-twitter-client-language': 'en'
			}

		guest_url = 'https://api.twitter.com/1.1/guest/activate.json'
		for _ in range(5):
			try:
				res = self.framework.request(
						guest_url,
						headers=self.header, 
						method='POST',
						timeout=5)
				break

			except Timeout:
				self.framework.error('Hit timeout on guest activation, trying again', 
						'util/osint/twitter', 'run_crawl')

			except Exception as e:
				self.framework.error(f"ConnectionError {e}.", 'util/osint/twitter', 'run_crawl')
				self.framework.error('Twitter is missed!', 'util/osint/twitter', 'run_crawl')
				return

		else:
			return

		self.header['x-guest-token'] = str(res.json()['guest_token'])

		if self._daterange == -1:
			self._search()
		else:
			self._datewise()
	
	def _search(self, date=None):
		search_url = f'https://twitter.com/i/api/2/search/adaptive.json'
		payload = {
			'simple_quoted_tweet': 'true',
			'q': self.q,
			'count': 20,
			'query_source': 'typed_query'
			}

		# Giving every page 3 max attempts 
		maxattempts = (self.max // 20) * 3
		attempts = 0

		page = 0

		while 20 * page <= self.max and attempts <= maxattempts:
			try:
				res = self.framework.request(
					    search_url, 
					    params=payload,
					    headers=self.header,
					    timeout=5)
				page += 1

			except Timeout:
				attemps += 1
				self.framework.error('Hit timeout on search endpoint, trying again', 
					    'util/osint/twitter', 'run_crawl')

			except Exception as e:
				self.framework.error(f"ConnectionError {e}.", 'util/osint/twitter', 'run_crawl')
				self.framework.error('Twitter is missed!', 'util/osint/twitter', 'run_crawl')
				return

			instructions = res.json()['timeline']['instructions']
			getCursorValue = lambda x: x['content']['operation']['cursor']['value'] 
			if page == 1:
				payload['cursor'] = getCursorValue(instructions[0]['addEntries']['entries'][-1])
			else:
				payload['cursor'] = getCursorValue(instructions[-1]['replaceEntry']['entry'])

			if date is None:
				self._json.update(res.json()['globalObjects']['tweets'])
			else:
				if self._json.get(date) is None:
					self._json[date] = res.json()['globalObjects']['tweets']
				else:
					self._json[date].update(res.json()['globalObjects']['tweets'])


	def _datewise(self):
		base = datetime.datetime.today()
		date_list = [(base - datetime.timedelta(days=x)).strftime('%Y-%m-%d') \
				for x in range(self._daterange+1)]
		suffix = lambda x,y: f" until:{x} since:{y}"
		orig_q = self.q\

		for x,y in zip(date_list, date_list[1:]):
			self.q = orig_q + suffix(x,y)
			self._search(date=x)

	def lookup_id(self, uid):
		'''Can accept upto 100 uids separated by ','
		'''
		if isinstance(uid, list):
			uid = ','.join(uid)

		user = self.framework.request(
				'https://api.twitter.com/1.1/users/lookup.json',
				params={'user_id': uid},
				headers=self.header).json()
		return user

	@property
	def tweets(self):
		if not self._verbose:
			if self._daterange == -1:
				self._tweets['all'] = []
				for tweet in self._json.values():
					tweet = tweet.get('text').replace('\n\n', '\n')
					self._tweets['all'].append(tweet)
			else:
				for day in self._json:
					self._tweets[day] = []
					for tweet in self._json[day]:
						tweet = self._json[day][tweet].get('text').replace('\n\n', '\n')
						self._tweets[day].append(tweet)
			return self._tweets
			
		else:
			return self._json



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from math import trunc

class main:
	def __init__(self):
		self.framework = main.framework

	def make_cite(self, url: 'URL string') -> 'cite':
		urlib = self.framework.urlib(url)
		path = urlib.path
		host = f"{urlib.scheme}://{urlib.netloc}"
		host_len = len(host)
		if path in ('', '/'):
			return host
		else:
			if host_len > 50:
				return f"{host} › ..."
		if '?' in path:
			path = path[:path.rfind('?')]
		if '.' in path:
			path = path[:path.rfind('.')]
		suffix = ''
		if path[:22] != urlib.path:
			suffix = '...'
		path = path[:-1] if path.endswith('/') else path
		path = path.replace('/', ' › ')[:22] + suffix
		cite = f"{host}{path}"
		return cite

	def remove_dups(self, res):
		urls = []
		new = []
		for i in res:
			a = self.urlib(i['a'].lower()).sub_service()
			if a not in urls:
				urls.append(a)
				new.append(i)
		return new

	def simple_merge(results) -> 'merging results based on quality of engines':
		engines_len = len(results)
		merged = []

		for i in range(len(min(results, key=len))):
			for e in range(engines_len):
				merged.append(results[e%engines_len].pop(0))


		for i in results:
			for j in i:
				merged.append(j)

		return merged

	def compute_count_consensus( 
			e: dict(type=list, help='list of search engines sorted by quality'),
			l: dict(type=int, help='number of results')) -> 'a list of numbers':
		x = len(e)
		o = {}
		for i in e:
			o[i] = trunc(l/x)
		l -= l - (l%x)
		if l != 0:
			if l < x:
				for i in range(l):
					o[e[i]] += 1
		return o



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

class main:

	def __init__(self, q, limit=1):
		""" duckduckgo.com search engine

			q          : Query for search
			limit      : Number of of pages
			count      : Number of of results
		"""
		self.framework = main.framework
		self.q = q
		self._pages = ''
		self.limit = 1
		self._links = []
		self._d_js_results = []
		self._d_js_xpath_name = {}
		self.d_js_xpath_name = {
				'results': '//div[@class="result results_links results_links_deep web-result "]',
				'results_content': './/a[@class="result__snippet"]',
				'results_title': './/a[@class="result__a"]',
				'results_a': './/a[@class="result__a"]/@href',
			}
		self.d_js_xpath = {
			self.d_js_xpath_name['results']: [
				self.d_js_xpath_name['results_content'],
				self.d_js_xpath_name['results_title'],
				self.d_js_xpath_name['results_a'],
			]
		}

	def run_crawl(self):
		page = 1
		set_page = lambda x: (x - 1) * 10 + 1
		payload = {'s': set_page(page), 'q': self.q, 'dc': 10, 'v': 'l', 'o': 'json'}
		duck_url = 'https://duckduckgo.com/html'
		self._pages = ''
		while True:
			self.framework.verbose(f"[DuckDuckGo] Searching in {page} page...", end='\r')
			try:
				req = self.framework.request(
					url=duck_url,
					params=payload)
			except Exception as e:
				self.framework.error(f"ConnectionError: {e}", 'util/duckduckgo', 'run_crawl')
				self.framework.error('DuckDuckGo is missed!', 'util/duckduckgo', 'run_crawl')
			else:
				if req.status_code != 200:
					self.framework.error(f"{req.status_code} Forbidden", 'util/duckduckgo', 'run_crawl')
					self.framework.error('DuckDuckGo is missed!', 'util/duckduckgo', 'run_crawl')
					break

				text = req.text
				self._pages += text

				if page == 1:
					vqd = re.search(r'"vqd" value="([\d\-]+)" />', text)
					if vqd:
						duck_url = f"https://links.duckduckgo.com/d.js"
						payload['vqd'] = vqd.group(1)
						payload['api'] = '/d.js'
						payload['s'] = set_page(page)
				else:
					try:
						self._d_js_results.extend(req.json()['results'])
					except Exception as e:
						pass
					else:
						pass

				page += 1
				if page >= self.limit:
					break

	@property
	def results(self):
		results = []
		parser = self.framework.page_parse(self._pages)
		xpath_results = parser.html_fromstring(self.d_js_xpath)
		root = xpath_results[self.d_js_xpath_name['results']]
		for i in range(len(root[self.d_js_xpath_name['results_a']])):
			a = root[self.d_js_xpath_name['results_a']][i]
			try :
				results.append({
					't': root[self.d_js_xpath_name['results_title']][i].text_content(),
					'a': a,
					'c': self.framework.meta_search_util().make_cite(a),
					'd': root[self.d_js_xpath_name['results_content']][i].text_content(),
				})
			except Exception as e:
				pass

		return results + [{'a': x['u'], 't': x['t'], 'd': x['a'], 'c': self.meta_search_util.make_cite(x['u'])}\
			for x in self._d_js_results]

	@property
	def pages(self):
		return self._pages

	@property
	def links(self):
		return self._links

	@property
	def links_with_title(self):
		parser = self.framework.page_parse(self._pages)
		parser.pclean
		results = [{ 'link': a, 'title': b } for a,b in parser.findall(r'''rel="nofollow" href="([^"]+)" class='result-link'>([^<]+)</a''')]

		return results

	@property
	def dns(self):
		return self.framework.page_parse(self._pages).get_dns(self.q)

	@property
	def emails(self):
		return self.framework.page_parse(self._pages).get_emails(self.q)

	@property
	def docs(self):
		return self.framework.page_parse(self._pages).get_docs(self.q, self.links)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import re

meta = {
	'name': 'Search public telegram groups',
	'author': 'Vikas Kundu',
	'version': '0.1',
	'description': 'Search the publicly listed telegram groups for juicy info like emails, phone numbers etc',
	'sources': ('telegramchannels.me','google', 'etools', 'bing', 'millionshort', 'duckduckgo'),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
		('limit', 1, False, 'Search limit(number of pages, default=1)', '-l', 'store', int),
		('count', 50, False, 'Number of results per page(min=10, max=100, default=50)', '-c', 'store', int),
		('thread', 2, False, 'The number of engine that run per round(default=2)', '-t', 'store', int),
		('engine', '', False, 'Engine names for search(default=telegramchannels.me)', '-e', 'store', str),
	),
    'examples': ('telegram -q <QUERY> -l 15 --output',)
}


LINKS = []
PAGES = ''

def search(self, name, q, q_formats, limit, count):
	global PAGES,LINKS
	engine = getattr(self, name)
	q = q_formats[f"{name}_q"] if f"{name}_q" in q_formats else q_formats['default_q']
	varnames = engine.__init__.__code__.co_varnames
	if 'limit' in varnames and 'count' in varnames:
		attr = engine(q, limit, count)
	elif 'limit' in varnames:
		attr = engine(q, limit)
	else:
		attr = engine(q)

	attr.run_crawl()
	LINKS += attr.links
	PAGES += attr.pages

def scrap(self,query,limit):
	global PAGES,LINKS	
	channel_links = []
	for page_no in range(1, limit + 1): # Scrapping all groups available
		self.verbose(f'[TELEGRAMCHANNELS] Searching in page {page_no}')
		try:
			req = self.request(f'https://telegramchannels.me/search?type=channel&page={page_no}&search={query}').text
		except Exception as e:
			self.error('Telegramchannels.me is missed!', 'telegram', 'scrap')
		else:
			if 'There are no media! Try another search!' in req:
				break
			else:
				channel_links += set(re.findall(r"https://telegramchannels\.me/channels/[\w]+", req)) # Find all channels

	pointer, total = 0, len(channel_links) # Variables for progress monitor
	# not using {channel_links.index(link)}/{len(channel_links)} as sometimes out of order iteration happens
	for link in set(channel_links): # Scraping channels individually
		pointer += 1
		self.verbose(f'[Telegramchannels.me ] Searching in channel {pointer}/{total}' )
		try:
			req = self.request(f'{link}').text
			LINKS += set(re.findall(r"t\.me/[\w]+", req))
			PAGES += req
		except Exception as e:
			self.error(f"Channel {link} is missed!", 'telegram', 'scrap')

def module_api(self):
	query = self.options['query']
	limit = self.options['limit']
	count = self.options['count']
	engines = self.options['engine'].split(',')
	output = {'group-links': [], 'handles': [], 'phone-numbers': []}
	q_formats = {
		'default_q': f"site:t.me/joinchat {query}",
		'millionshort_q': f'site:t.me/joinchat "{query}"',
		'qwant_q': f'site:t.me/joinchat {query}'
	}

	self.thread(search, self.options['thread'], engines, query, q_formats, limit, count, meta['sources'])
	
	scrap(self, query, limit)
	global LINKS #To avoid the error: local variable 'LINKS' referenced before assignment.
	LINKS = list(set(LINKS)) 
	output['group-links'] = list(self.reglib().filter(r"https?://([\w\-\.]+\.)?t\.me/joinchat/", LINKS))
	
	output['group-links'] += list(self.reglib().filter(r"t\.me/[\w]+", LINKS)) # Output for scraping sepertely

	output['handles'] = [i for i in set(re.findall(r"@[\w]+", PAGES)) if i not in ['@media', '@keyframes', '@font'] ]
	
	phone_set = [str(i) for i in set(re.findall(r"(\+?\d{1,3}[- ]?)?(\d{10}\s)", PAGES)) ]
	# The regex is that: (it may start with a + or digits ranging from 1 to 3 optionally followed by space or -) or
	# ... (simply 10 digits followed by a space)
	
	for i in phone_set:	# Cleaning up the mobile numbers format
		i = re.sub(r"[^\d\+-]", '', i) # Remove all chars except digits, + and -
		output['phone-numbers'].append(i)
		
	self.save_gather(output, 'search/telegram', query, output=self.options.get('output'))
	return output

def module_run(self):
	self.alert_results(module_api(self))



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import flask
from flask import request, jsonify, Response

framework = None
app = flask.Flask('OWASP Maryam')

@app.route('/', methods=['GET', 'POST'])
def home():
	page = Response('<pre>current pages:<br>/api/modules => running modules<br>/api/framework => framework commands</pre>', headers={'X-Content-Type-Options': '*'}) 
	return page


@app.route('/api/', methods=['GET', 'POST'])
def api():
	page = '<pre>current pages:<br>/api/modules => running modules<br>/api/framework => framework commands'\
		   '<br><b>/api/modules?_module=<module-name>&options[short or long]...</b>'\
		   '<br>["meta"]: api metadata'\
		   '<br>["meta"]["error"]: error messages. None if no error occurs'\
		   '<br>["meta"]["command"]: input command'\
		   '<br>["output"]: module output'\
		   '<br>["output"]["running_errors]: error messages that occurs during running the module'\
		   '<br><b>/api/framework?command=<command></b>'\
		   '<br>["meta"]: api metadata'\
		   '<br>["meta"]["error"]: error messages. None if no error occurs'\
		   '<br>["meta"]["command"]: input command'\
		   '</pre>'
	return page

@app.route('/api/framework')
def api_framework():
	error = None
	command = None
	page = {'meta': {'error': error, 'command': command}}
	if 'command' in request.args:
		invalid_commands = ['workspaces', 'set', 'unset', 'history', 'report', 'update']
		command = request.args['command']
		if command != '':
			if command.split(' ')[0] in invalid_commands:
				framework.onecmd(command)
				command = request.args['command']
			else:
				error = 'Invalid command.'
		else:
			error = 'No command specified.'
	else:
		error = 'no command specified.'
	page['meta']['command'] = command
	page['meta']['error'] = error
	return jsonify(page)

@app.route('/api/modules')
def api_modules():

	page = {'meta': {'error': None, 'command': None}, 'output': {}}
	# If no module specified
	args_dict = request.args.to_dict()
	if '_module' not in args_dict:
		page['meta']['error'] = 'No module specified.'
		return jsonify(page)
	module_name = args_dict.pop('_module')
	# If module doesn't exist
	if module_name not in framework._loaded_modules:
		page['meta']['error'] = f"Module name '{module_name}' not found."
		return jsonify(page)
	if args_dict == {}:
		page['meta']['error'] = 'No option specified.'
		return jsonify(page)
	module = framework._loaded_modules[module_name]
	options = module.meta['options']
	true_options = ('true', 'on', 'yes', '1', True)
	framework.options = {}
	# Add framework options
	if 'output' in args_dict:
		if args_dict['output'] in true_options:
			framework.options['output'] = True
		else:
			framework.options['output'] = False
	else:
		framework.options['output'] = False

	# Setting options
	for option in options:
		option_name = option[0]
		option_required = option[2]
		option_type = option[6]
		option_name_short = option[4][1:]
		option_action = option[5]
		if option_name in args_dict:
			option_value = args_dict[option_name]
		elif option_name_short in args_dict:
			option_value = args_dict[option_name_short]
		else:
			option_value = option[1]

		if option_action == 'store':
			if isinstance(option_value, option_type):
				framework.options[option_name] = option_value
			else:
				try:
					option_value = option_type(option_value)
				except:
					page['meta']['error'] = f"Need {option_type}. got invalid type for {option_name}."
					return jsonify(page)
				else:
					framework.options[option_name] = option_value
		else:
			if option_value in true_options:
				framework.options[option_name] = True
			else:
				framework.options[option_name] = option_value
	try:
		output = framework.mod_api_run(module_name)
	except Exception as e:
		framework.print_exception()
		output = False
	if output == False:
		page['meta']['error'] = 'Something went wrong.'
	else:
		page['output'] = output
		if page['output']['running_errors'] != []:
			page['meta']['error'] = 'Runtime error.'
	page['meta']['command'] = framework.options
	return jsonify(page)

@app.errorhandler(404)
def page_not_found(e):
    return "<pre>404</pre>", 404

def run_app(core_obj, host='127.0.0.1', port=1313):
	global framework
	framework = core_obj
	app.run(host=host, port=port, debug=True)

@app.after_request
def add_header(response):
	response.headers['Origin'] = 'https://cors-anywhere.herokuapp.com'
	response.headers['X-Content-Type-Options'] = '*'
	response.headers['Access-Control-Allow-Origin'] = '*'
	response.headers['Access-Control-Allow-Origin'] = '*'
	response.headers['Access-Control-Allow-Methods'] = "HEAD, GET, POST, PUT, DELETE, OPTIONS, PATCH"
	response.headers['Access-Control-Allow-Headers'] = "*"
	response.headers['Access-Control-Allow-Credentials'] = "true"
	return response



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__VERSION__ = '2.5.3'

import sys

# Import framework
from maryam.core import initial

def cui(args):
	core = initial.initialize
	help_menu = 'usage: maryam [-h] [-e execute] [-s start] [-v]'\
				  '\nmaryam -h\n'\
				  '\noptional arguments:'\
				  '\n	  -h, --help   show this help message and exit'\
				  '\n	  -e execute   execute a command and exit'\
				  '\n	  -s start     run a command without exit'\
				  '\n	  -v           show version and exit'
	if args:
		option = args.pop(0)
		mode = 'run'
		if option in ['-v', '--version']:
			print(f"OWASP Maryam V.{__VERSION__}")
			exit(0)
		if option == '-e':
			core = core('execute')
			core.onecmd(' '.join(args))
			exit(0)
		elif option == '-s':
			core = core('run')
			core.onecmd(' '.join(args))
		elif option == 'run':
			core = core('run')
		else:
			print(help_menu)
			exit(0)
	else:
		core = core('run')

	while True:
		try:
			core.cmdloop()
			break
		except KeyboardInterrupt:
			print('\n[!] Use exit command to exit')
		except Exception as e:
			raise e

if __name__ == '__main__':
	args = [f'"{arg}"' if ' ' in arg else arg for arg in sys.argv[1:]]
	cui(args)



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


class main:
	def __init__(self, q, limit=1, count=10):
		""" bing.com search engine
			q     : Query for search
			limit : Number of pages
			count : Number of results
		"""
		self.framework = main.framework
		self.q = q
		self.agent = 'Mozilla/5.0 (X11; Linux x86_64; rv:100.0) Gecko/20100101 Firefox/100.0'
		self.xpath_name = {
			'results': '//li[@class="b_algo"]',
			'results_content_and_cite': './/div[@class="b_caption"]',
			'results_links': './/h2/a',
		}
		self.xpath = {
			self.xpath_name['results']: [
				self.xpath_name['results_content_and_cite'],
				self.xpath_name['results_links']
			]
		}
		self.url = 'https://www.bing.com/search'
		self._pages = ''
		self._first_page = ''
		self.limit = limit + 1
		self.count = count

	def run_crawl(self):
		page = 1
		set_page = lambda x: (x - 1) * 10 + 1
		payload = {'count': self.count, 'first': set_page(page), 'form': 'QBLH', 'pq': self.q.lower(), 'q': self.q}
		max_attempt = 0
		while True:
			self.framework.verbose(f"[BING] Searching in {page} page...", end='\r')
			try:
				req = self.framework.request(
					url=self.url,
					params=payload,
					headers={'user-agent': self.agent},
					allow_redirects=True)
			except Exception as e:
				self.framework.error(f"ConnectionError: {e}", 'util/engines/bing', 'run_crawl')
				max_attempt += 1
				if max_attempt == self.limit:
					self.framework.error('Bing is missed!', 'util/engines/bing', 'run_crawl')
					break
			else:
				self._pages += req.text
				if page == 1:
					self._first_page += req.text
				page += 1
				payload['first'] = set_page(page)
				if page >= self.limit or 'title="Next page"' not in req.text:
					break

	# TODO: Finish it
	@property
	def bing_card(self):
		card_xpath_name = {
			'card': '//div[@class="lite-entcard-main"]',
			'card_names': '//div[@class="l_ecrd_hero"]',
			'card_description': './/span[@class="l_ecrd_txt_pln"]',
			'card_description_link': './/a[@class=" l_ecrd_txt_lnk l_ecrd_txt_hover"]',
			'card_social_nets': '//div[@class="l_ecrd_hero"]/div[@class="l_ecrd_webicons"]/div/a',
			'card_facts': '//div[@class="l_ecrd_qfi4_fcts"]/div[@class="l_ecrd_qfi4_fct"]',
		}
		xpath = {
			card_xpath_name['card']: [
				card_xpath_name['card_description'],
				card_xpath_name['card_description_link'],
				card_xpath_name['card_social_nets'],
				card_xpath_name['card_facts'],
			]
		}
		parser = self.framework.page_parse(self._first_page)
		xpath_results = parser.html_fromstring(xpath)
		output = {'content': '', 'source': '', 'networks': [], 'facts': []}
		root = xpath_results[card_xpath_name['card']]
		if root[card_xpath_name['card']]:
			output['content'] = root[card_xpath_name['card_names']][0].text_content().strip()
			output['content'] += '\n' + root[card_xpath_name['card_description']][0].text_content().strip()
			output['source'] = root[card_xpath_name['card_description_link']][0].get('href')
			output['networks'] = []
			output['facts'] = []
			for i in root[card_xpath_name['card_social_nets']]:
				output['networks'].append(i.get('href'))
			for i in root[card_xpath_name['card_facts']]:
				output['facts'].append(i.text_content())
		return output

	@property
	def results(self):
		parser = self.framework.page_parse(self._pages)
		xpath_results = parser.html_fromstring(self.xpath)
		results = []
		if not xpath_results:
			return results
		root = xpath_results[self.xpath_name['results']]
		for i in range(len(root[self.xpath_name['results_content_and_cite']])):
			try:
				link = root[self.xpath_name['results_links']][i]
				title = link.text_content()
				a = link.get('href')
				c_and_c = root[self.xpath_name['results_content_and_cite']][i]
				desc = c_and_c.xpath('.//p')[0].text_content().strip()
				# cite = c_and_c.xpath('.//div[@class="b_attribution"]//cite')[0].text_content().strip()
				cite = self.framework.meta_search_util().make_cite(a)
				result = {
					't': title,
					'a': a,
					'c': cite,
					'd': desc,
				}
				results.append(result)
			except:
				pass
		return results

	@property
	def pages(self):
		return self._pages

	@property
	def links(self):
		return [x['a'] for x in self.results]

	@property
	def dns(self):
		return self.framework.page_parse(self._pages).get_dns(self.q, self.links)

	@property
	def emails(self):
		return self.framework.page_parse(self._pages).get_emails(self.q)

	@property
	def docs(self):
		return self.framework.page_parse(self._pages).get_docs(self.q, self.links)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from random import choice

class main:

	agents = ['Googlebot/2.1 ( http://www.googlebot.com/bot.html)',
			'Debian APT-HTTP/1.3 (0.8.10.3)',
			'Chromium/9.0.595.0 Chrome/9.0.595.0 Safari/534.13',
			'Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Ubuntu/10.04'
			'Mozilla/5.0 (compatible; 008/0.83; http://www.80legs.com/webcrawler.html) Gecko/2008032620',
			'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.0.5) Gecko/20060719 Firefox/1.5.0.5',
			'Opera/9.80 (Windows NT 5.2; U; ru) Presto/2.5.22 Version/10.51',
			'Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; .NET CLR 2.0.50727)',
			'Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)',
			'Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm)',
			'Googlebot/2.1 (+http://www.googlebot.com/bot.html)',
			'msnbot/1.1 (+http://search.msn.com/msnbot.htm)',
			'Mozilla/5.0 (compatible; Yahoo! Slurp; http://help.yahoo.com/help/us/ysearch/slurp)',
			'YahooSeeker/1.2 (compatible; Mozilla 4.0; MSIE 5.5; yahooseeker at yahoo-inc dot com ;',
			'http://help.yahoo.com/help/us/shop/merchant/)',
			'Mozilla/5.0 (compatible; YandexBot/3.0; +http://yandex.com/bots)',
			'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-gb) AppleWebKit/528.4+ (KHTML, like Gecko) Version/4.0dp1 Safari/526.11.2',
			'Mozilla/4.0 (compatible; MSIE 5.22; Mac_PowerPC)',
			'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; pl) Opera 8.54',
			'Mozilla/5.0 (Windows NT 5.1; U; en) Opera 8.51',
			'Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.8.1) Gecko/20061208 Firefox/5.0 Opera 11.11',
			'Opera/8.01 (Macintosh; U; PPC Mac OS; en)',
			'Opera/8.01 (Windows NT 5.1; U; de)',
			'Opera/9.80 (Windows NT 5.2; U; en) Presto/2.6.30 Version/10.63',
			'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; ko; rv:1.9.1b2) Gecko/20081201 Firefox/3.1b2',
			'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; he; rv:1.9.1b4pre) Gecko/20100405 Firefox/3.6.3plugin1',
			'Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US; rv:1.8.1.12pre) Gecko/20080122 Firefox/2.0.0.12pre',
			'Mozilla/5.0 (X11; FreeBSD i686) Firefox/3.6']

	lynx = ['Lynx/2.8.5rel.1 libwww-FM/2.15FC SSL-MM/1.4.1c OpenSSL/0.9.7e-dev',
					'Lynx/2.8.5dev.16 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/0.9.7a',
					'Lynx/2.8.9dev.8 libwww-FM/2.14 SSL-MM/1.4.1 GNUTLS/3.4.9',
					'Lynx/2.8.5rel.1 libwww-FM/2.14 SSL-MM/1.4.1 GNUTLS/1.0.16',
					'Lynx/2.8.4rel.1 libwww-FM/2.14 SSL-MM/1.4.1 GNUTLS/0.8.6',
					'Lynx/2.8.4rel.1 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/0.9.6b',
					'Lynx/2.8.8pre.4 libwww-FM/2.14 SSL-MM/1.4.1 GNUTLS/2.12.23',
					'Lynx/2.8.8dev.3 libwww-FM/2.14 SSL-MM/1.4.1',
					'Lynx/2.8.7dev.4 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/0.9.8d',
					'Lynx/2.8.9dev.11 libwww-FM/2.14 SSL-MM/1.4.1 GNUTLS/3.5.6']

	phone = ['WeatherReport/1.0.2 CFNetwork/485.13.9 Darwin/11.0.0',
					'WeatherReport/1.2.2 CFNetwork/485.13.9 Darwin/11.0.0',
					'WeatherReport/1.2.2 CFNetwork/485.12.7 Darwin/10.4.0',
					'WeatherReport/1.2.1 CFNetwork/485.12.7 Darwin/10.4.0',
					'WeatherReport/1.2.0 CFNetwork/485.12.7 Darwin/10.4.0',
					'WeatherReport/1.2.1 CFNetwork/485.13.9 Darwin/11.0.0',
					'WeatherReport/1.2.2 CFNetwork/485.13.8 Darwin/11.0.0'
					'WeatherReport/1.2.2 CFNetwork/467.12 Darwin/10.3.1',
					'WeatherReport/1.0.2 CFNetwork/485.12.7 Darwin/10.4.0',
					'WeatherReport/1.2.1 CFNetwork/467.12 Darwin/10.3.1']

	server = ['Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko Firefox/11.0 (via ggpht.com GoogleImageProxy)',
					'Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)',
					'Mozilla/5.0 (compatible; Baiduspider/2.0; +http://www.baidu.com/search/spider.html)',
					'Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm)',
					'YahooMailProxy; https://help.yahoo.com/kb/yahoo-mail-proxy-SLN28749.html',
					'Mozilla/5.0 (compatible; MJ12bot/v1.4.5; http://www.majestic12.co.uk/bot.php?+)',
					'Mozilla/5.0 (compatible; Yahoo! Slurp; http://help.yahoo.com/help/us/ysearch/slurp)',
					'Mozilla/5.0 (compatible; MegaIndex.ru/2.0; +http://megaindex.com/crawler)',
					'Mozilla/5.0 (compatible; AhrefsBot/5.2; +http://ahrefs.com/robot/)',
					'Apache/2.4.7 (Unix) OpenSSL/1.0.1e PHP/5.4.22 mod_perl/2.0.8-dev Perl/v5.16.3 (internal dummy connection)']

	@property
	def get(self):
		return choice(self.agents)



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

meta = {
	'name': 'Find Emails',
	'author': 'Saeed',
	'version': '1.1',
	'description': 'Search in open-sources to find emails.',
	'sources': ('bing', 'pastebin', 'google', 'metacrawler',
				'baidu', 'startpage', 'duckduckgo', 'hunter', 'github', 'keyserver',),
	'options': (
		('query', None, True, 'Domain name or company name', '-q', 'store', str),
		('limit', 3, False, 'Search limit(number of pages, default=3)', '-l', 'store', int),
		('count', 50, False, 'number of results per page(min=10, max=100, default=50)', '-c', 'store', int),
		('engines', None, True, 'Search engine names. e.g bing,google,..', '-e', 'store', str),
		('key', None, False, 'Give a valid hunter API key. Limit for free plan is 10 results', '-k', 'store', str),
		('thread', 2, False, 'The number of engine that run per round(default=2)', '-t', 'store', int),
	),
	'examples': ('email_search -q microsoft.com -e bing --output',
		'email_search -q owasp.org -e google,bing,yahoo -l 20 -t 3 --output')
}

EMAILS = []

def search(self, name, q, q_formats, limit, count):
	global EMAILS
	engine = getattr(self, name)
	eng = name
	varnames = engine.__init__.__code__.co_varnames
	q = q_formats[f"{eng}"] if f"{eng}" in q_formats else q_formats['default']
	if 'limit' in varnames and 'count' in varnames:
		attr = engine(q, limit, count)
	elif 'limit' in varnames and 'key' in varnames:
		key = q.split('&api_key=')[1]
		k_q = q.split('&api_key=')[0]
		if key == 'None':
			self.error('-k <API KEY> is required for hunter', 'email_search', 'search')
			return
		else:
			attr = engine(k_q, key, limit)
	elif 'limit' in varnames:
		attr = engine(q, limit)
	else:
		attr = engine(q)
	
	attr.run_crawl()
	EMAILS.extend(attr.emails)

def module_api(self):
	query = self.options['query']
	domain = self.options['query'].replace('@', '')
	urlib = self.urlib(domain)
	domain = self.urlib(domain).netroot if '/' in domain else domain
	limit = self.options['limit']
	count = self.options['count']
	engines = self.options['engines'].split(',')
	key = self.options['key']
	q_formats = {
		'default': f'"%40{domain}"',
		'ask': f"%40{domain}",
		'hunter': f"{domain}&api_key={key}",
		'github': domain,
		'keyserver': domain
	}
	self.thread(search, self.options['thread'], engines, query, q_formats, limit, count, meta['sources'])
	output = {'emails': list(set(EMAILS))}
	self.save_gather(output, 'osint/email_search', domain,\
		output=self.options['output'])
	return output

def module_run(self):
	self.alert_results(module_api(self))



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import re

meta = {
	'name': 'Trello Search',
	'author': 'Vikas Kundu',
	'version': '0.5',
	'description': 'Search public trello boards for data and show the results.',
	'sources': ('google', 'etools', 'bing', 'millionshort', 'duckduckgo'),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
		('limit', 1, False, 'Search limit(number of pages, default=1)', '-l', 'store', int),
		('count', 50, False, 'Number of results per page(min=10, max=100, default=50)', '-c', 'store', int),
		('thread', 2, False, 'The number of engine that run per round(default=2)', '-t', 'store', int),
		('engine', 'google', False, 'Engine names for search(default=google)', '-e', 'store', str),
	),
    'examples': ('trello -q <QUERY> -l 15 --output',)
}


LINKS = []
PAGES = ''

def search(self, name, q, q_formats, limit, count):
	global PAGES,LINKS
	engine = getattr(self, name)
	q = q_formats[f"{name}_q"] if f"{name}_q" in q_formats else q_formats['default_q']
	varnames = engine.__init__.__code__.co_varnames
	if 'limit' in varnames and 'count' in varnames:
		attr = engine(q, limit, count)
	elif 'limit' in varnames:
		attr = engine(q, limit)
	else:
		attr = engine(q)

	attr.run_crawl()
	LINKS += attr.links
	PAGES += attr.pages

def module_api(self):
	query = self.options['query']
	limit = self.options['limit']
	count = self.options['count']
	engines = self.options['engine'].split(',')
	output = {'links': [], 'usernames': [], 'handles': []}
	q_formats = {
		'default_q': f"site:trello.com {query}",
		'millionshort_q': f'site:trello.com "{query}"',
		'qwant_q': f'site:trello.com {query}'
	}

	self.thread(search, self.options['thread'], engines, query, q_formats, limit, count, meta['sources'])
	
	output['links'] = list(self.reglib().filter(r"https?://([\w\-\.]+\.)?trello\.com/", list(set(LINKS))))
	output['handles'] = [ i for i in set(re.findall(r"@[\w]+", PAGES)) if i not in ['@keyframes', '@media','@font']  ]

	output['usernames'] = [ i.replace('(','@') for i in set(re.findall(r"\([\w]{3,}\)", PAGES)) if query in i  ]
	output['usernames'] = [ i.replace(')','') for i in output['usernames'] ]

	#using re directly instead of reglib().filter because when PAGES is passed a list,
	#it becomes useless for searching trello usernames. See: https://ibb.co/Rz182Bj
	
	self.save_gather(output, 'search/trello', query, output=self.options.get('output'))
	return output

def module_run(self):
	self.alert_results(module_api(self))



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
import os
from wordcloud import WordCloud
import matplotlib.pyplot as plt

class main:	
	def __init__(self):	
		self.framework = main.framework

	def _remove_url(self, data):
		return re.sub(r"(http|ftp|https)://([\w_-]+(?:(?:\.[\w_-]+)+))([\w.,@?^=%&:/~+#-]*[\w@?^=%&/~+#-])?", '', data)	
	
	def plot_wcloud(self, title, docs: 'documents', form: 'documet form. e.g html', limit: 'number of words', 
	without_punc=True, remove_stopwords=False, should_show=True):
		docs = str(docs)
		docs = self._remove_url(docs)
		tf = self.framework.tf_histogram(docs, form, without_punc)

		if remove_stopwords:
			tf.remove_stopwords()
		bow = tf._counter(limit)
		cloud_data = ' '.join(i[0] for i in bow)

		if not cloud_data:
			self.framework.error('NoDataToPrintError.', 'util/iris/word_cloud', 'plot_wcloud')
			return False

		wcd = WordCloud().generate(cloud_data)
		plt.imshow(wcd, interpolation='bilinear')
		plt.axis("off")
		plt.title(title)
		filename = os.path.join(self.framework.workspace,title.replace(' ','_')+'.png')
		plt.savefig(filename, format="png")
		if should_show:
			plt.show()

		return filename




"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

meta = {
	'name': 'Etools Search',
	'author': 'Saeed',
	'version': '0.1',
	'description': 'Search your query in the etools.ch and show the results.',
	'sources': ('etools',),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
	),
	'examples': ('etools -q <QUERY>',)
}

def module_api(self):
	query = self.options['query']
	run = self.etools(query)
	run.run_crawl()
	results = run.results
	output = {'results': results}
	self.save_gather(output, 'search/etools', query, output=self.options['output'])
	return output

def module_run(self):
	self.search_engine_results(module_api(self))



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

meta = {
	'name' : 'Regex Web Search',
	'author' : 'Saeed',
	'version' : '0.6',
	'description' : 'Search to find keywords, emails, usernames, errors, meta tags and regex on the page/pages.',
	'options' : (
		('domain', None, True, 'Domain string', '-d', 'store', str),
		('regex', None, True, 'Regex or string for search in the pages', '-r', 'store', str),
		('more', False, False, 'Extract more information from the pages', '--more', 'store_true', bool),
		('limit', 1, False, 'Scraper depth level(default=1)', '-l', 'store', int),
		('debug', False, False, 'debug the scraper', '--debug', 'store_true', bool),
		('thread', 1, False, 'The number of links that open per round', '-t', 'store', int),
	),
	'examples': ('crawl_pages -d <DOMAIN> -r "https?://[A-z0-9\./]+"\
	 --output', 'crawl_pages -d <DOMAIN> --limit 2 --more')
}

def module_api(self):
	domain = self.options['domain']
	regex = self.options['regex']
	try:
		re.compile(regex)
	except Exception as e:
		self.error(e, 'crawl_pages', 'module_api')
		return
	scrap = self.web_scrap(domain, self.options['debug'], self.options['limit'], self.options['thread'])
	scrap.run_crawl()
	pages = scrap.pages
	category_pages = scrap.category_pages
	# Output flag
	flag = 0
	resp = {}
	methods = []
	resp[regex] = {}
	out = []
	## Regular Expression Search ##
	###############################
	if regex:
		reg_find = {}
		self.alert('Regex search:')
		regex_cmp = re.compile(regex)
		for page in category_pages:
			founds = regex_cmp.findall(category_pages[page])
			self.output(f"\tRegex {regex} search at {page}:")
			resp[regex].update({page : founds})
			for i in founds:
				if i not in reg_find.keys():
					reg_find[i] = 1
				else:
					reg_find[i] = reg_find[i] + 1

			for i in reg_find:
				self.output(f"\t\tFound '{i}' Repeated {reg_find[i]} times", 'G')
			if reg_find:
				if not flag:
					flag = 1
			reg_find = {}

		if not flag:
			self.output(f"\tNo response\n")


	if not self.options['more']:
		self.save_gather(resp, 'footprint/crawl_pages', domain, output=self.options['output'])
		return resp
	resp[regex]['more'] = {}

	page_parse = self.page_parse(pages)

	get_meta = page_parse.get_metatags
	resp[regex]['more']['meta'] = get_meta
	for key,attrs in enumerate(get_meta):
		self.alert(f"META {key}:")
		for attr_name in attrs:
			print(attr_name, attrs[attr_name])
			self.output(f"\t{attr_name} : {attrs[attr_name]}")

	## EMAILS ##
	############
	emails = page_parse.all_emails
	if emails != []:
		self.alert('Emails:')
		for i in emails:
			self.output(f"\t'{i}'", 'G')
	resp[regex]['more']['emails'] = emails

	## SOCIAL NETWORK ##
	####################
	usernames = page_parse.get_networks
	users = []
	self.alert('Social Networks:')
	for net in usernames:
		lst = list(set(usernames[net]))
		if lst != []:
			self.alert(net)
			for link in lst:
				users.append(link)
				self.output(f"\t{str(link)}", 'G')
	resp[regex]['more']['usernames'] = users

	self.save_gather(resp, 'footprint/crawl_pages', domain, [regex, 'more'], output=self.options['output'])
	return resp

def module_run(self):
	module_api(self)



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import os
import concurrent.futures
from socket import gethostbyname

from maryam.core.basedir import BASEDIR

meta = {
	'name': 'DNS Brute Force',
	'author': 'Saeeddqn',
	'version': '1.1',
	'description': 'DNS brute force attack, supports concurrency.',
	'comments': ('The wordlist option can be an url',),
	'options': (
		('domain', None, True, 'Domain name without https?://', '-d', 'store', str),
		('count', None, False, 'Number of payloads len(max=count of payloads). default is max',
							 '-c', 'store', int),
		('wordlist', os.path.join(BASEDIR, 'data', 'dnsnames.txt'), False, 
			   'wordlist address. default is dnsnames.txt in data folder', '-w', 'store', str),
		('thread', 8, False, 'The number of links that open per round(default=8)', '-t', 'store', int),
		('wordlists', False, False, 'List of most common DNS wordlists', '-l', 'store_true', bool),
		('ips', False, False, 'Show ip addresses', '-i', 'store_true', bool),
	),
	'examples': ('dnsbrute -d <DOMAIN> --output',
				'dnsbrute -d <DOMAIN> -w <WORDLIST>')
}

HOSTNAMES = []
LISTS = {"https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/DNS/subdomains-top1million-5000.txt": 'small',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/DNS/fierce-hostlist.txt": 'small',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/DNS/namelist.txt": 'small',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/DNS/subdomains-top1million-20000.txt": 'medium',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/DNS/subdomains-top1million-110000.txt": 'large',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/DNS/deepmagic.com-prefixes-top50000.txt": 'large',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/DNS/shubs-subdomains.txt": 'large'
}

def thread(self, function, hostname, wordlist, thread_count):
	threadpool = concurrent.futures.ThreadPoolExecutor(max_workers=thread_count)
	futures = (threadpool.submit(function, self, hostname, word) for word in wordlist if not word.startswith("#"))
	counter = 1
	for _ in concurrent.futures.as_completed(futures):
		print(f"Checking payload {counter}, hits: {len(HOSTNAMES)}", end='\r')
		counter += 1
	print(os.linesep)

def request_checker(self, hostname, word):
	hostname = f"{word}.{hostname}".lower()
	try:
		req = self.request(f"http://{hostname}")
	except Exception as e:
		if 'timed out' in str(e.args):
			self.verbose(f"{hostname} => Timed out", 'O')
		else:
			self.debug(f"{list(e.args)[0]}:'{hostname}'", 'O')
	else:	
		if self.options['ips']:
			try:
				ip = gethostbyname(hostname)
			except:
				ip = '-'
			hostname = f"{hostname} : {ip}"
		HOSTNAMES.append(hostname)
		self.output(f"{hostname}{' '*50}", 'G')

def remote_list(self, addr):
	req = self.request(addr)
	headers = req.headers
	keys = list(map(str.lower, list(headers.keys())))
	cond1 = 'content-type' in keys and 'text/plain' in str(headers).lower()
	cond2 = 'githubusercontent.com' in addr.lower()
	cond3 = '<' not in req.text
	if cond1 or cond2 or cond3:
		wordlist = req.text.split('\n')
		if '' in wordlist:
			wordlist.pop(wordlist.index(''))
		return wordlist
	else:
		self.error(f"{addr} value is not text/plain", 'dnsbrute', 'remote_list')
	return []

def module_api(self):
	domain = self.options['domain']
	urlib = self.urlib(domain)
	domain = urlib.sub_service('http')
	domain = domain.replace('www.', '')
	hostname = urlib.netloc
	wordlist = self.options['wordlist']
	count = self.options['count']
	if '://' in wordlist:
		dlist = remote_list(wordlist)
	else:
		dlist = self._is_readable(wordlist)
		if dlist:
			dlist = dlist.read().split()
		else:
			dlist = []

	dlen = len(dlist)
	if not count:
		count = dlen
	else:
		count = dlen if count > dlen else count

	self.heading(f"Starting DNS brute force with {count} payload", level=0)
	thread(self, request_checker, hostname, dlist[:count], self.options['thread'])

	self.save_gather({'hostnames': HOSTNAMES}, 'footprint/dnsbrute', hostname, output=self.options['output'])
	return HOSTNAMES

def module_run(self):
	if self.options["wordlists"]:
		self.alert('common DNS wordlists')
		self.table(LISTS.items(), header=('list', 'scale'))
		return
	self.alert_results(module_api(self))



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

meta = {
	'name': 'Iris_Cluster',
	'author': 'Shaad',
	'version': '0.1',
	'description': 'Get Iris Search result and clustered results for your query',
	'required': ('kneed', 'mlxtend, numpy, sklearn'),
	'options': (
			('query', None, True, 'Query string', '-q', 'store', str),
		),
	'examples': ('iris_cluster -q <QUERY>')
}


def module_api(self):
	query = self.options['query']
	output_option_value = self.options['output']
	output = {}
	mode = self._mode
	# Computing iris search result
	self._mode = 'api_mode'
	iris_search_result = self.opt_proc('iris', args=f'-q "{query}" --api', output='web_api')
	# Computing cluster results
	clusterer = self.cluster(iris_search_result)
	output = {'results': clusterer.perform_clustering()}

	self._mode = mode
	# Resetting options for iris_search_module
	self.options = {}
	self.options['query'] = query 
	self.options['output'] = output_option_value

	self.save_gather(output, 'iris/iris_cluster', query, output=self.options['output'])
	return output

def module_run(self):
	output = module_api(self)['results']

	print('\n\nCLUSTER RESULT: ')
	for index, title in enumerate(output):
	    print('\n')
	    print(f"CLUSTER {index+1}")
	    print(f"TITLE: {title}")
	    print('  '+'\n  '.join(output[title]))



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re


class main:

    def __init__(self, q, limit=1):
        """
        darksearch.io search engine

        q          : Query for search
        limit      : Number of pages
        """

        self.framework = main.framework
        self.q = q
        self._pages = ''
        self.limit = limit
        self._links = []
        self._json = []
        self.darksearch = 'darksearch.io'
        self._json_links = []

    def run_crawl(self):
        urls = [f"https://{self.darksearch}/api/search?query={self.q}&page={i}" for i in range(1, self.limit+1)]
        max_attempt = len(urls)
        for url in range(max_attempt):
            self.framework.verbose(f"[DARKSEARCH] Searching in {url} page...")
            try:
                req = self.framework.request(url=urls[url], allow_redirects=True)
            except Exception as e:
                self.framework.error('ConnectionError', 'util/darksearch', 'run_crawl')
                max_attempt -= 1
                if max_attempt == 0:
                    self.framework.error('Darksearch is missed!', 'util/darksearch', 'run_crawl')
                    break

            if req.json()['data'] is None:
                break
            self._pages += req.text
            self._json.append(req.json())

    @property
    def pages(self):
        return self._pages

    @property
    def json(self):
        return self._json

    @property
    def links(self):
        for page in self.json:
            results = page.get('data', {})
            if not results:
                return {}
            self._links.extend(x.get('link') for x in results)
        return self._links

    @property
    def json_links(self):
        for page in self.json:
            results = page.get('data', {})
            if not results:
                return {}
            self._json_links.extend(results)
        return self._json_links



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class main:

	def __init__(self, q, limit=2):
		""" startpage.com search engine

			q 		  : Query for search
			limit	  : Number of pages
		"""
		self.framework = main.framework
		self.q = self.framework.urlib(q).quote
		self.limit = limit
		self.xpath_name = {
			'results': '/html/body/div[2]/div/div[2]/div[1]/div[2]/div[1]/div/section[2]/div',
			'results_content': './/div[1]/p',
			'results_links': './/div[1]/div[2]/a',
			'results_title': './/div[1]/div[2]/a/h3'
		}
		self.xpath = {
			self.xpath_name['results']: [
				self.xpath_name['results_content'],
				self.xpath_name['results_links'],
				self.xpath_name['results_title']
			]
		}
		self._pages = ''
		self.startpage = 'startpage.com'

	def run_crawl(self):
		url = f"https://{self.startpage}/sp/search"

		headers = {
			'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:90.0) Gecko/20100101 Firefox/90.0',
			'Content-Type': 'application/x-www-form-urlencoded'
		}

		data = {
				'query': self.q
		}

		self.framework.verbose(f"[STARTPAGE] Searching in {self.startpage}...")

		for i in range(self.limit):
			data['page'] = i

			try:
				req = self.framework.request(
						url=url,
						method='POST',
						headers=headers,
						data=data
				)
			except Exception as e:
				self.framework.error(f"ConnectionError {e}.", 'util/startpage', 'run_crawl')
				self.framework.error('Startpage is missed!', 'util/startpage', 'run_crawl')
				return

			else:
				page = req.text
				self._pages += page

				if '> Next <' not in page:
					break


	@property
	def results(self):
		parser = self.framework.page_parse(self._pages)
		xpath_results = parser.html_fromstring(self.xpath)
		results = []
		if not xpath_results:
			return results
		root = xpath_results[self.xpath_name['results']]
		for i in range(len(root[self.xpath_name['results_links']])):
			link = root[self.xpath_name['results_links']][i].get('href')
			title = root[self.xpath_name['results_title']][i].text_content().strip()
			desc = root[self.xpath_name['results_content']][i].text_content().strip()
			cite = self.framework.meta_search_util().make_cite(link)
			result = {
				't': title,
				'a': link,
				'c': cite,
				'd': desc,
			}
			results.append(result)
		return results

	@property
	def pages(self):
		return self._pages

	@property
	def dns(self):
		return self.framework.page_parse(self._pages).get_dns(self.q)

	@property
	def emails(self):
		return self.framework.page_parse(self._pages).get_emails(self.q)

	@property
	def docs(self):
		return self.framework.page_parse(self._pages).get_docs(self.q)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class main:

	def __init__(self, q, key, limit=10):
		""" hunter.io search engine

			q 		  : query for search
			key 	  : hunter.io api key
			limit	  : Number of pages
		"""
		self.framework = main.framework
		self.q = q
		self.limit = limit
		self.key = key 
		self._pages = ''
		self._json_pages = ''
		self.hunter_api = f"https://api.hunter.io/v2/domain-search?domain={self.q}&api_key={self.key}"                                                                                                               
		self.acceptable = False
	def run_crawl(self):
		self.framework.verbose('[HUNTER] Searching in hunter...')
		try:
			req = self.framework.request(self.hunter_api)
		except:
			self.framework.debug('ConnectionError', 'util/hunter', 'run_crawl')
			self.framework.error('Hunter is missed!', 'util/hunter', 'run_crawl')
			return
		self._pages = req.text
		self._json_pages = req.json()

		# Key validation
		if 'errors' in self._json_pages:
			code = self._json_pages.get('errors')[0]['details']
			self.framework.error(f"failed with error: {code}", 'util/hunter', 'run_crawl')
			self.acceptable = False
			return

		# Request validation
		if not self._json_pages.get('data').get('emails'):
			if self._json_pages.get('meta').get('results') == 0:
				self.framework.verbose('[HUNTER] no results found')
			else:
				self.framework.verbose('[HUNTER] request was not accepted!')
		else:
			self.acceptable = True

	@property
	def pages(self):
		return self._pages
	
	@property
	def json_pages(self):
		return self._json_pages

	@property
	def emails(self):
		return self.framework.page_parse(self._pages).get_emails(self.q)

	@property
	def json_emails(self):
		emails = []
		if self.acceptable:
			for x in range(self.limit):
				emails.append(self._json_pages['data']['emails'][x]['value'])
			return emails
		return []
	
	@property
	def dns(self):
		return self.framework.page_parse(self.pages).get_dns(self.q)



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

meta = {
	'name': 'GitHub Search',
	'author': 'Aman Singh',
	'version': '1.0',
	'description': 'Search your query in the GitHub and show the results.',
	'sources': ('google', 'etools', 'bing', 'yahoo', 'millionshort', 'duckduckgo', 'github'),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
		('limit', 1, False, 'Search limit(number of pages, default=1)', '-l', 'store', int),
		('count', 50, False, 'Number of links per page(min=10, max=100, default=50)', '-c', 'store', int),
		('engine', 'google,github', False, 'Engine names for search(default=google, github)', '-e', 'store', str),
		('thread', 2, False, 'The number of engine that run per round(default=2)', '-t', 'store', int),
	),
    'examples': ('github -q <QUERY> -l 15 -e etools,bing,qwant --output',)
}

LINKS = []
PAGES = ''
USERS = []
REPO = []
EMAILS = []

def search(self, name, q, q_formats, limit, count):
	global PAGES, LINKS, USERS, REPO, EMAILS
	engine = getattr(self, name)
	eng = name
	q = q_formats[f"{name}"] if f"{name}" in q_formats else q_formats['default']
	varnames = engine.__init__.__code__.co_varnames
	if 'limit' in varnames and 'count' in varnames:
		attr = engine(q, limit, count)
	elif 'limit' in varnames:
		attr = engine(q, limit)
	else:
		attr = engine(q)
	if eng == 'github':
		run = self.github(q)
		run.run_crawl()
		USERS += run.users
		REPO += run.repositories
		EMAILS = run.emails
	else:
		attr.run_crawl()
		LINKS += attr.links
		PAGES += attr.pages

def module_api(self):
	query = self.options['query']
	limit = self.options['limit']
	count = self.options['count']
	engines = self.options['engine'].split(',')
	output = {'repositories': [], 'blogs': [], 'usernames': [], 'emails': set()}
	q_formats = {
		'default': f"site:github.com {query}",
		'millionshort': f'site:github.com "{query}"',
		'qwant': f'site:github.com {query}',
		'github': f'{query}'
	}

	self.thread(search, self.options['thread'], engines, query, q_formats, limit, count, meta['sources'])

	explore = self.page_parse(PAGES).get_networks
	output['blogs'] = list(set(explore['Github site']))
	bactery = ['marketplace', 'pulls', 'explore', 'issues',\
				'notifications', 'account', 'settings', 'login',\
				'about', 'pricing', 'site']
	for user in set(explore['Github'][1:]):
		user = user.split('/')[1]
		if user not in bactery:
			output['usernames'].append(user)
	for user in USERS[1:]:
		user = user.split('/')[1]
		output['usernames'].append(user)

	for link in self.reglib().filter(r"https://(www\.)?github\.com/[\w-]{1,39}/[\w\-\.]+", list(set(LINKS))):
		repo = re.search(r"(https://(www\.)?github\.com/([\w-]{1,39})/[\w\-\.]+)", link)
		if repo.group(3) not in bactery:
			repo = repo.group(0)
			if repo not in output['repositories']:
				output['repositories'].append(repo)
	for link in REPO:
		output['repositories'].append(link)
	
	output['emails'] = EMAILS
	self.save_gather(output,
	 'search/github', query, output=self.options.get('output'))
	return output

def module_run(self):
	self.alert_results(module_api(self))



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

meta = {
	'name': 'CRT Search',
	'author': 'Saeed',
	'version': '0.2',
	'description': 'Search your query in the crt.sh and show the results.',
	'sources': ('crt',),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
	),
	'examples': ('crt -q <QUERY>',)
}

def module_api(self):
	query = self.options['query']
	run = self.crt(query)
	run.run_crawl()
	output = run.json_page
	self.save_gather({'certificates': output}, 'search/crt', query, output=self.options['output'])
	return output

def module_run(self):
	output = module_api(self)
	items = []
	for itm in output:
		i_d = f"https://crt.sh/?caid={itm.get('id')}"
		entry_timestamp = itm.get('entry_timestamp').split('T')[0]
		not_before = itm.get('not_before').split('T')[0]
		not_after = itm.get('not_after').split('T')[0]
		nvalue = itm.get('name_value')
		issuer = f"https://crt.sh/?caid={itm.get('issuer_ca_id')}"
		if '\n' in nvalue:
			for line in nvalue.split('\n'):
				items.append((i_d, entry_timestamp, not_before, not_after, line, issuer))
			continue
		items.append((i_d, entry_timestamp, not_before, not_after, nvalue, issuer))
	self.table(items, header=['crt.sh ID', 'Logged at', 'Not Before', 'Not After', 'Maching Identities', 'Issuer Name'], linear=True, sep='_')



#!/usr/bin/env python3
"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class main:

	def __init__(self, q, limit=10):
		""" keyserver.ubuntu.com search engine

			q 		  : query for search
			limit	  : Number of pages
		"""
		self.framework = main.framework
		self.q = q
		self.limit = limit
		self._pages = ''
		self._json_pages = ''
		self.keyserver_api = f"https://keyserver.ubuntu.com/pks/lookup?search=@{self.q}&op=index"
		self.acceptable = False

	def run_crawl(self):
		self.framework.verbose('[KEYSERVER] Searching in keyserver...')
		try:
			req = self.framework.request(self.keyserver_api)
		except:
			self.framework.debug('ConnectionError', 'util/keyserver', 'run_crawl')
			self.framework.error('Keyserver is missed!', 'util/keyserver', 'run_crawl')
			return
		self._pages += req.text
		return self.framework.page_parse(self._pages).get_emails(self.q)

	@property
	def pages(self):
		return self._pages
	
	@property
	def json_pages(self):
		return self._json_pages

	@property
	def emails(self):
		return self.framework.page_parse(self._pages).get_emails(self.q)

	@property
	def dns(self):
		return self.framework.page_parse(self.pages).get_dns(self.q)




"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import json

meta = {
	'name': 'Find Web Entry Points',
	'author': 'Saeed',
	'version': '0.1',
	'description': 'Crawl web pages to find entry points(inputs, urls with param).',
	'options': (
		('domain', None, True, 'Domain string', '-d', 'store', str),
		('debug', False, False, 'debug the scraper', '--debug', 'store_true', bool),
		('limit', 1, False, 'Scraper depth level', '-l', 'store', int),
		('thread', 1, False, 'The number of links that open per round', '-t', 'store', int),
	),
	'examples': ('entry_points -d <DOMAIN>', 'entry_points -d <DOMAIN> --output --debug -l 10 -t 3')
}

def module_api(self):
	domain = self.options['domain']
	run = self.web_scrap(domain, self.options['debug'], self.options['limit'], self.options['thread'])
	run.run_crawl()
	get = run.query_links
	parser = self.page_parse(run.pages)
	forms = parser.get_forms
	urls = {}
	urlib = self.urlib('null')
	for link in get:
		params = urlib.self_params(link)
		urls.update(params)
	output = {'params': urls, 'forms': forms}
	self.save_gather(output, 'footprint/entry_points', domain, output=self.options['output'])
	return output

def module_run(self):
	output = module_api(self)
	self.alert('FORMS')
	print(json.dumps(output['forms'], indent=4))
	print(json.dumps(output['params'], indent=4))



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from urllib import parse as urlparse
import re
from socket import gethostbyname, gethostbyaddr

class main:

	def __init__(self, url):
		""" url parser

			url 	: url string for parse
		"""
		self.url = url

	def join(self, urjoin):
		return urlparse.urljoin(self.url, urjoin)

	def parse(self):
		return urlparse.urlparse(self.url)

	def unparse(self, urparse):
		return urlparse.urlunparse(urparse)

	def sub_service(self, serv=None, ifany=False):
		'''Add protocol to url or replace it or clean it'''
		urparse = re.split(r'://', self.url)
		if not serv:
			# Clean protocol
			if len(urparse) == 2:
				url = ''.join(urparse[1:])
			else:
				url = ''.join(urparse)
		else:
			# Add protocol
			serv = re.sub(r'://', '', serv)
			if len(urparse) == 2:
				if ifany:
					return self.url
				del urparse[0]
				url = f"{serv}://{''.join(urparse)}"
			else:
				url = f"{serv}://{urparse[0]}"
		self.url = url
		return url

	def check_urlfile(self, file):
		reg = re.compile(r"\."+file+r"[^\w]")
		reg2 = re.compile(r"\."+file+r"[^\w]?$")
		if reg.search(self.url) or reg2.search(self.url):
			return True
		return False

	@property
	def quote(self):
		if '%' not in self.url:
			self.url = urlparse.quote(self.url)
		return self.url

	@property
	def quote_plus(self):
		if '%' not in self.url:
			self.url = urlparse.quote_plus(self.url)
		return self.url

	@property
	def unquote(self):
		self.url = urlparse.unquote(self.url)
		return self.url

	@property
	def unquote_plus(self):
		self.url = urlparse.unquote_plus(self.url)
		return self.url

	@property
	def ip(self):
		if re.match(r"^\d+.\d+.\d+.\d+$", self.url):
			return self.url
		else:	
			loc = self.netloc
			return gethostbyname(self.netloc)

	@property
	def host(self):
		return gethostbyaddr(self.parse().netloc)[0]

	@property
	def scheme(self):
		return self.parse().scheme

	@property
	def netloc(self):
		return self.parse().netloc

	@property
	def netroot(self):
		loc = self.parse().netloc
		# Replace subdomains
		reg = re.search(r"^[A-z0-9\-.]+\.([A-z0-9\-]+\.[A-z0-9]+)$", loc)
		if reg:
			loc = reg.group(1)
		return loc

	@property
	def path(self):
		return self.parse().path

	@property
	def query(self):
		return self.parse().query

	@property
	def params(self):
		return self.parse().params

	@property
	def fragment(self):
		return self.parse().fragment

	def self_params(self, url):
		url = urlparse.unquote(url)
		queries = urlparse.urlparse(url).query
		page = url.replace('?'+queries, '')
		params = {}
		params[page] = {}
		if not queries:
			return {}

		if '&' in queries:
			queries = queries.split('&')
		else:
			queries = [queries]
		for query in queries:
			if not query:continue
			query = query.split('=')+['']
			name=query[0]
			value=query[1]
			params[page][name] = value
		return params



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

meta = {
	'name': 'Article Search',
	'author': 'Kaushik',
	'version': '0.1',
	'description': 'Search for scientific papers and articles from Google Scholar \
			, Arxiv, and Pubmed',
	'sources': ('scholar', 'arxiv', 'pubmed'),
	'options': (
		('query', None, True, 'Search query for papers or articles', '-q', 'store', str),
		('limit', 15, False, 'Max result count (default=15)', '-l', 'store', int),
		('exclude medical', False, False, 'Exclude medical results', '-e', 'store_true', bool),
	),
	'examples': ('article_search -q <query>', 'article_search -q <query> --output')
}

def module_api(self):
	q = self.options['query']
	no_medical = self.options['exclude medical']
	limit = self.options['limit']//3 if no_medical else self.options['limit']//4

	scholar = self.scholar(q, limit)
	scholar.run_crawl()
	results = scholar.results

	# core_ac = self.core_ac(q, limit)
	# core_ac.run_crawl()
	# results.extend(core_ac.results)

	arxiv = self.arxiv(q, limit)
	arxiv.run_crawl()
	results.extend(arxiv.results)

	if not no_medical:
		pubmed = self.pubmed(q, limit)
		pubmed.run_crawl()
		results.extend(pubmed.results)

	output = {'results': results}

	self.save_gather(output, 'osint/article_search', q, output=self.options['output'])
	return output

def module_run(self):
	self.search_engine_results(module_api(self))



# modules/iris/topicmodeling.py
# # TESTING The Main Function =
# topicmodeling -i mixed.json -t json -m all-distilroberta-v1
# topicmodeling -i mixed.json -t json -s -v -m all-distilroberta-v1
# topicmodeling -i mixed.json -t json -s -m all-distilroberta-v1
# topicmodeling -i mixed.json -t json -v -m all-distilroberta-v1
# topicmodeling -i testdataset.csv -t csv -m all-mpnet-base-v2
# topicmodeling -i testdataset.csv -t csv -s -v -m all-mpnet-base-v2
# topicmodeling -i testdataset.csv -t csv -s -m all-mpnet-base-v2
# topicmodeling -i testdataset.csv -t csv -v -m all-mpnet-base-v2
# # TESTING The Reporting & API Function =
# topicmodeling -i mixed.json -t json -m all-distilroberta-v1 --output
# report json testreport iris/topicmodeling
# topicmodeling -i mixed.json -t json -m all-distilroberta-v1 --api
# # TESTING The Function: Search Topics Using a Keyword (Top2Vec) =
# topicmodeling -i mixed.json -t json -m all-distilroberta-v1 -k music
# Note: download the dataset for testing from https://github.com/keamanansiber/Maryam/tree/master/notebooks
# Hatma Suryotrisongko


meta = {
	'name': 'Topic Modeling',
	'author': 'Hatma Suryotrisongko',
	'version': '0.1.0',
	'description': 'Topic Modeling Algorithms.',
	'required': ('dask', 'scikit-learn', 'umap', 'bertopic', 'gensim', 'top2vec'),
	'options': (
		('inputfile', None, True, 'Input file that contains the data', '-i', 'store', str),
		('filetype', None, True, 'File type: csv/json', '-t', 'store', str),
		('keyword', None, False, 'Search keyword: ', '-k', 'store', str),
		('showcharts', None, False, 'Show charts?', '-s', 'store_true', bool),
		('verbose', None, False, 'Verbose output?', '-v', 'store_true', bool),
		('pretrained_model', None, True, 'model for embedding', '-m', 'store', str),
	),
	'examples': ('topicmodeling -i mixed.json -t json -k music -s True -v False -m all-mpnet-base-v2')
}


def module_api(self):

	run = self.topic(self.options['inputfile'], self.options['filetype'], self.options['keyword'], self.options['showcharts'], self.options['verbose'])
	run.run_sklearn_cluster_kmeans(self.options['pretrained_model'], self.options['showcharts'], self.options['verbose'])

	results = run.run_topic_modeling_bertopic(self.options['pretrained_model'], self.options['verbose'])
	self.output("\n\nResults = \n")
	self.output( results )

	output = {'results': results.to_json(orient="records") }

	inputfile = self.options['inputfile']
	self.save_gather(output, 'iris/topicmodeling', inputfile, output=self.options['output'])

	if self.options['keyword'] is not None:
		self.output(" keyword = " + self.options['keyword'])
		run.run_search_topics_top2vec(self.options['keyword'], self.options['showcharts'], self.options['verbose'])

	return output


def module_run(self):
	output = module_api(self)
	self.output("\n\nOutput = \n")
	self.output( output )


#!/usr/bin/env python3
"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
import os
import sys
import json
import shutil
import socket
import threading
from requests import get
from time import sleep, time
from base64 import b64decode
from hashlib import md5

from maryam.core.basedir import BASEDIR

meta = {
	'name': 'Dark Web Crawler',
	'author': 'Kaushik',
	'version': '0.1',
	'description': 'A Dark Web Crawler that uses Best First Snowball Sampling',
	'options': (
		('query', None, True, 'Comma separated query strings', '-q', 'store', str),
		('depth', 1, False, 'Scraper depth level', '-d', 'store', int),
		('threads', 5, False, 'The number of workers', '-t', 'store', int),
	),
	'examples': ('dark_web_crawler -q <DOMAIN>',)
}

def _search(self, query, df):
    retriever = self.retriever(ngram_range=(1, 2), min_df=0, max_df=10, stop_words='english')
    df = retriever.filter_pages(df)
    retriever.fit(df)
    best = retriever.predict(query)
    best_indexes = list(best.keys())
    best_scores = list(best.values())
    return best_scores, list(df.iloc[inx] for inx in best_indexes)

def calculate_rank(self, query, data):
	"""Sort data by similarity to query"""
	import pandas as pd

	pages = list(data.values())
	df = pd.DataFrame(pages, columns=['pages'])
	df['pages'] = df['pages'].apply(lambda x:[x])
	scores, pages = _search(self, query, df)
	return scores, list(map(lambda x: x.tolist()[0][0], pages))

def assign_ranks(self):
	"""Assign ranks and save result"""
	global locked, \
			depth, \
			pos_q, \
			neg_q, \
			data,  \
			RESULTS

	locked = True
	scores, sorted_pages = calculate_rank(self, ' '.join(query), data)
	data = {}

	try:
		sorted_urls = list(map(lambda x: x[:x.index(' || ')], sorted_pages))
	except Exception:
		self.output(f"\r{' '*cols()}\r|| ERROR {sorted_pages[i]}", color='R')
		locked = False
		return 1

	for url in reversed(sorted_urls):
		# If url has children shift all children to beginning of q
		if url in children:
			pos_q = children[url][0] + list(set(pos_q)-set(children[url][0]))
			neg_q = children[url][1] + list(set(neg_q)-set(children[url][1]))

	clrline()
	separator = '\n' + ' ' * 24
	toprint = separator.join(sorted_urls[:5])
	self.output(f'Best result at depth {depth}: {toprint}\n', color='Y')

	if depth not in RESULTS:
		RESULTS[depth] = []

	for i in range(min(len(sorted_pages), 10)):
		try:
			a = sorted_pages[i].index(' || ')
			headers = sorted_pages[i][a+4:a+56].strip().replace('\n','')
			RESULTS[depth].append(f"{sorted_urls[i]} - {headers}")
		except Exception:
			self.output(f"\r{' '*cols()}\r|| ERROR {sorted_pages[i]}", color='R')

	depth += 1
	locked = False

def main(self):
	"""Actual searcher"""
	global visited,    \
		data,		   \
		children,	   \
		locked,		   \
		depth,		   \
		should_exit,   \
		pos_q,		   \
		neg_q,		   \
		page_hashes,   \
		blacklist,     \
		cols

	q_len = lambda: len(pos_q) + len(neg_q)
	print_status = lambda: self.output((f"\r{' '*cols()}\rDepth: {depth}  Queued: {q_len()}  "
		f"Searched: {len(data)}  Current: {curr[:70]+'...' if len(curr)>75 else curr}"), linesep=False, color='G')

	while depth <= maxdepth and not should_exit:
		start = time()
		while q_len() == 0 or locked:
			sleep(1)
			if time() - start > 9:
				if len(data) > 0:
					assign_ranks(self)
				should_exit = True
				return None

		if len(pos_q) > 0:
			curr = pos_q.pop(0)
			timeout = 10
		else:
			curr = neg_q.pop(0)
			timeout = 8

		print_status()

		fqdn_end = curr.index('onion') + 5

		# if subdirs of site have been visited more than 20 times then dont visit site
		# again
		if curr not in visited:
			visited.append(curr)
		else:
			continue

		try:
			tosearch = get(
				curr, 
				proxies = {
					'http':'socks5h://localhost:9050',
					'https':'socks5h://localhost:9050'
				},
				timeout = timeout
			).text
			sleep(1)
		except Exception as err:
			self.output(f"\r{' '*cols()}\r{curr} took too long to respond", color="R")

			# If a url is unresponsive remove all its subdirectories from queues
			pos_q = list(filter(lambda x: not x.startswith(curr[:fqdn_end]), pos_q))
			neg_q = list(filter(lambda x: not x.startswith(curr[:fqdn_end]), neg_q))

			continue

		if len(tosearch) < 30 or any(list(map(lambda x: x in tosearch, blacklist))):
			# print(f'\r{" "*cols()}\r{red}{curr} too small or contains blacklisted token(s){reset}')
			continue

		# There are often redirects to the same page by different urls.
		# Confirm that hash of current page is unique before proceeding
		page_hash = md5(tosearch.encode('utf-8')).hexdigest()
		if page_hash not in list(page_hashes.values()):
			page_hashes[curr] = page_hash
		else:
			continue

		score, _ = calculate_rank(self, ' '.join(query), {'_': tosearch})

		if score[0] > 0.01 * (depth + 1) and not should_exit:
			pp = self.page_parse(tosearch)
			pp.remove_html_tags

			# Doing this because after ranking pages we can grab url easily
			data[curr] = curr + ' || ' + pp.page

			complete_urls = set(re.findall(url_regex, tosearch))
			unique_complete = complete_urls - set(visited) - (set(pos_q).union(set(neg_q)))

			subdirs = set(
				map(
					lambda x: curr[:fqdn_end] + x if x.startswith('/') else curr[:fqdn_end] + '/' + x, 
					re.findall(subdir_regex, tosearch)
				)
			)
			unique_subdirs = subdirs - set(visited) - (set(pos_q).union(set(neg_q)))

			unique = unique_complete.union(unique_subdirs)
			# else:
			#	  unique = unique_complete

			# Positive set contains all the url that contain one or more keywords in them
			positive_set = set(
				filter(lambda x: any(list(keyword in x.lower() for keyword in query)), unique)
			)

			# Everything else goes in negative_set
			negative_set = unique - positive_set

			pos_q.extend(list(positive_set))
			neg_q.extend(list(negative_set))

			if len(unique) > 0:
				if len(negative_set) > 0:
					clrline()
					print('\n'.join(list(negative_set)[:30]))
				if len(positive_set) > 0:
					clrline()
					print('\n'.join(positive_set))

				children[curr] = [list(positive_set), list(negative_set)]

			else:
				self.output(f"\r{' '*cols()}\rNo links found in {curr}", color = "R")

		if len(data) > 15*(depth):
			assign_ranks(self)

def module_api(self):
	global RESULTS,    \
		visited,       \
		data,		   \
		maxdepth,      \
		children,	   \
		locked,		   \
		depth,		   \
		should_exit,   \
		pos_q,		   \
		neg_q,		   \
		page_hashes,   \
		blacklist, 	   \
		query,         \
		cols,          \
		clrline,       \
		url_regex,     \
		subdir_regex

	RESULTS = {}
	maxdepth = self.options['depth']

	cols = lambda: shutil.get_terminal_size().columns
	clrline = lambda : self.output('\r' + ' '*cols() + '\r', linesep=False)

	query = list(map(lambda x: x.lower(), self.options['query'].split(',')))

	banned_types = "|".join([
		'png',
		'svg',
		'gif',
		'jpg',
		'mp4',
		'js',
		'css',
		'xml',
		'rss',
		'webp',
		'jpeg',
		'woff',
		'eot',
		'ttf',
		'zip'
	])

	url_regex = rf'((?:https?:\/\/)\w+.onion\/?(?!\S+?(?:{banned_types}))[\w\/\-%\.\?]*\/?)'
	subdir_regex = r'<a href="(\/?\S+(?:html|php)\?.*?)"'

	visited = []
	page_hashes = {}
	children = {}
	data = {}

	depth = 1
	locked = False
	should_exit = False

	# Must do this if you want colours to work on windows
	iswin = sys.platform.startswith('win')
	if iswin:
		os.system('color')

	sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	sock.settimeout(1)
	host = socket.getaddrinfo('127.0.0.1', 9050)
	status = sock.connect_ex(host[-1][4])
	if status != 0:
		self.error('Tor proxy not found on port 9050, aborting', 'dark_web_crawler', 'main')
		self.error(
			'Refer to https://2019.www.torproject.org/docs/debian.html.en for installation', 
			'dark_web_crawler', 
			'main'
		)
		should_exit = True
		exit(0)

	# Load blacklist
	blacklist = []
	with open(os.path.join(BASEDIR, 'data', 'blacklist.txt')) as f:
		for line in f:
			blacklist.append(b64decode(line).decode())

	if any(list(word in query for word in blacklist)):
		self.output('Blacklist disabled because query contains blacklisted token', color='Y')
		blacklist = []

	# Use search engine results for query as starting urls for crawler
	self.alert('Grabbing results from Phobos')
	neg_q = []
	for i in range (1,4):
		url = f"http://phobosxilamwcg75xt22id7aywkzol6q6rfl2flipcqoc4e4ahima5id.onion/search?query={'+'.join(query)}&p={i}"
		headers = {
			'Referer': url
		}
		phobos_page = get(
			url, 
			proxies={
				'http':'socks5h://localhost:9050',
				'https':'socks5h://localhost:9050'
			},
			headers = headers,
			timeout=10
		).text
		neg_q.extend(re.findall('<a class="titles" href="(.*?)"', phobos_page))

	self.alert('Grabbing results from Ahmia')
	url = f"https://ahmia.fi/search/?q={'+'.join(query)}"
	ahmia_page = get(url).text
	neg_q.extend(re.findall(r'redirect_url=(.*?)">', ahmia_page))

	pos_q = []

	threadcount = self.options['threads']
	threads=[]
	# Anything above four causes tor to ban us within depth 2
	for _ in range(threadcount):
		threads.append(threading.Thread(target=main, args=(self,)))
		threads[-1].start()
	
	while depth <= maxdepth and not should_exit:
		sleep(2)

	return RESULTS

def module_run(self):
	module_api(self)
	self.output('Waiting for outstanding requests to finish.')
	sleep(8)
	print()
	for depth in RESULTS.keys():
		self.output(f"Depth {depth} :")
		for result in RESULTS[depth]:
			self.output('\t' + result)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from datetime import datetime

class main:

	def __init__(self, query, count=50, search='posts', username='', subreddit='', before='', after='', score='' ):
		""" Search in reddit using the pushshift api
		
			query		: query to seach for
			count		: no. of results to show
			search		: search in posts or comments
			username	: reddit username
			subreddit	: reddit subreddit
			before		: show data only before this timestamp
			after		: show data only after this timestamp
			score		: score of the posts or comments

		"""
		self.framework = main.framework
		self.query = query
		self.count = count
		self.search = search
		self.username = username
		self.subreddit = subreddit
		self.before = before
		self.after = after
		self.score = score
		self._usernames = set() # To ensure unique usernames
		self._links = set() # To ensure unique links
	
	def convert_to_epoch_time(self, date):
		try:		
			date_time_obj = datetime.strptime(date, '%d/%m/%y %H:%M:%S')
		except ValueError:
			self.framework.error("Reddit_pushshift timestamp format is wrong. It should be '%d/%m/%y %H:%M:%S' i.e. 7/6/21 12:0:0. This parameter is skipped!", \
			'util/reddit_pushshift', 'convert_to_epoch_time')
			return ''
		else:
			return int(date_time_obj.timestamp())
				
	def run_crawl(self):
		self.framework.verbose('[Reddit Pushshift] Searching...')
		self.before = self.convert_to_epoch_time(self.before) if self.before else self.before
		self.after = self.convert_to_epoch_time(self.after) if  self.after else self.after
		base_url = 'https://api.pushshift.io/reddit/submission/search?html_decode=true' if self.search == 'posts' \
		else 'https://api.pushshift.io/reddit/comment/search?html_decode=true'
		query_url = f"{base_url}&q={self.query}&author={self.username}&subreddit={self.subreddit}&size={self.count}&before={self.before}&after={self.after}&score={self.score}"
		
		try:
			response = self.framework.request(query_url, timeout=20) # Increased timeout for date based queries which take long
		except Exception as e:
			self.framework.error('Reddit_pushshift is missed!', 'util/reddit_pushshift', 'search')
			return False
		else:	
			data = response.json()
			for i in data['data']:
				self._usernames.add(i['author'])
				self._links.add(f"https://www.reddit.com{i['permalink']}")

	@property
	def usernames(self):
		return list(self._usernames)
			
	@property
	def links(self):
		return list(self._links)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

class main:

	def __init__(self, q):
		""" ahmia.fi search engine
			
			q 		  : query for search
		"""
		self.framework = main.framework
		self.q = self.framework.urlib(q).quote
		self._pages = ''
		self._links = []
		self.ahmia = 'ahmia.fi'

	def run_crawl(self):
		url = f"https://{self.ahmia}/search/?q={self.q}"
		try:
			req = self.framework.request(url=url)
		except:
			self.framework.print_exception()
			self.framework.error('Ahmia is missed!', 'util/ahmia', 'run_crawl')
		self._pages = req.text
			
	@property
	def pages(self):
		return self._pages

	@property
	def links(self):
		return self.framework.page_parse(self._pages).findall(r'redirect_url=(.*?)">')



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

meta = {
	'name': 'Cloud Storage',
	'author': 'Vikas Kundu',
	'version': '0.1',
	'description': 'Search for the query in online storage services like GoogleDrive, OneDrive, Dropbox,\
	 Amazon S3, Box and show the results.',
	'sources': ('google', 'etools', 'bing', 'duckduckgo'),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
		('limit', 1, False, 'Search limit(number of pages, default=1)', '-l', 'store', int),
		('count', 50, False, 'Number of links per page(min=10, max=100, default=50)', '-c', 'store', int),
		('engine', 'bing', False, 'Engine names for search(default=google)', '-e', 'store', str),
		('thread', 2, False, 'The number of engine that run per round(default=2)', '-t', 'store', int),
	),
	'examples': ('cloud_storage -q <QUERY> -l 15 --output',)
}

LINKS = []
PAGES = ''

def search(self, name, q, q_formats, limit, count):
	global PAGES,LINKS
	engine = getattr(self, name)
	q = q_formats[f"{name}_q"] if f"{name}_q" in q_formats else q_formats['default_q']
	varnames = engine.__init__.__code__.co_varnames
	if 'limit' in varnames and 'count' in varnames:
		attr = engine(q, limit, count)
	elif 'limit' in varnames:
		attr = engine(q, limit)
	else:
		attr = engine(q)

	attr.run_crawl()
	LINKS += attr.links
	PAGES += attr.pages

def module_api(self):
	query = self.options['query']
	limit = self.options['limit']
	count = self.options['count']
	engine = self.options['engine'].split(',')

	sites = ['drive.google.com', 'dl.dropboxusercontent.com', '1drv.ms', 'box.com/s', 's3.amazonaws.com']
	output = { 'links': [] }

	for site_url in sites:
		q_formats = {
			'default_q': f'site:{site_url} {query}',
			'millionshort_q': f'site:{site_url} "{query}"',
			'qwant_q': f'site:{site_url} {query}'
		}
	
		self.thread(search, self.options['thread'], engine, query, q_formats, limit, count, meta['sources'])
			
		output['links'] += list( self.reglib().filter(r"https?://([\w\-\.]+\.)?"\
		+ site_url.replace('.','\.')+"/", list( set(LINKS) ) ) ) #escaping . for regex search using replace()

	self.save_gather(output, 'osint/cloud_storage', query, output=self.options.get('output'))
	return output

def module_run(self):
	self.alert_results(module_api(self))



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class main:
    def __init__(self, u, limit=100):
        """ urlscan.io scanner

        u       : The url for scanning
        limit   : Number of pages
        """
        self.framework = main.framework
        self.url = u
        self.limit = limit
        self._ips = []
        self._domains = []
        self.urlscan_api = f"https://urlscan.io/api/v1/search/?q=domain:{self.url}"

    def run_crawl(self):
        domains = []
        self.framework.verbose('[URLSCAN] Searching in urlscan.io...')
        try:
            req = self.framework.request(self.urlscan_api)
            result_json = req.json()
        except:
            self.framework.error('[URLSCAN] Connection Error')
            return
        else:
            if not result_json['total']:
                self.framework.verbose('[URLSCAN] Not yet in our database.')
                return
            elif 'results' not in result_json:
                self.framework.error(f"failed with an error: {result_json['description']}", 'util/urlscan', 'run_crawl')
                return
            else:
                for results in result_json['results']:
                    domains.append(results['task']['domain'])
                    domains.append(results['page']['domain'])
                for i in range(len(domains)):
                    if self.url in domains[i]:
                        self._domains.append(domains[i])

    @property
    def dns(self):
        return self._domains

    @property
    def get_ips(self):
        return self._ips


"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

class main:

	def __init__(self, q, limit=2, count=100):
		""" yahoo.com search engine

			framework : core attribute
			q 		  : query for search
			limit	  : Number of pages
			count	  : Number of links
		"""
		self.framework = main.framework
		self.q = self.framework.urlib(q).quote
		self.limit = limit
		self.count = count
		self.agent = 'Mozilla/5.0 (X11; Linux x86_64; rv:86.0) Gecko/20100101 Firefox/86.0'
		self.url = 'https://search.yahoo.com/search'
		self._pages = ''
		self.xpath_name = {
			'results': "//div[contains(concat(' ', normalize-space(@class), ' '), ' Sr ')]",
			'results_a': './/h3/a/@href',
			'results_title': './/h3/a',
			'results_content': './/div[contains(@class, "compText")]'
		}
		self.xpath = {
			self.xpath_name['results']: [
				self.xpath_name['results_content'],
				self.xpath_name['results_title'],
				self.xpath_name['results_a']
			]
		}

	def run_crawl(self):
		page = 1
		set_page = lambda x: (x - 1) * 10 + 1
		payload = {'p': self.q, 'b': set_page(page), 'pz': self.count, 'fl': 1}
		max_attempt = 0
		while True:
			self.framework.verbose(f"[YAHOO] Searching in {page} page...")
			try:
				req = self.framework.request(
					url=self.url,
					params=payload,
					headers={'user-agent': self.agent},
					allow_redirects=True)
			except:
				self.framework.error('ConnectionError', 'util/yahoo', 'run_crawl')
				max_attempt += 1
				if max_attempt == self.limit:
					self.framework.error('Yahoo is missed!', 'util/yahoo', 'run_crawl')
					break
			else:
				if 'unsafe-url">Next<' not in req.text or page == self.limit:
					self._pages += req.text
					break
				page += 1
				payload['b'] = set_page(page)
				self._pages += req.text

	@property
	def pages(self):
		return self._pages

	@property
	def results(self):
		parser = self.framework.page_parse(self._pages)
		xpath_results = parser.html_fromstring(self.xpath)
		results = []
		if not xpath_results:
			return results
		root = xpath_results[self.xpath_name['results']]
		for i in range(len(root[self.xpath_name['results_a']])):
			a = re.search(r"RU=(https?%3a%2f%2f[^/]+)/", root[self.xpath_name['results_a']][i])
			if a:
				a = a.group(1)
			else:
				a = root[self.xpath_name['results_a']][i]
			urlib = self.framework.urlib(a)
			a = urlib.unquote
			cite = self.framework.meta_search_util().make_cite(a)
			result = {
				't': root[self.xpath_name['results_title']][i].text_content(),
				'a': a,
				'c': cite,
				'd': root[self.xpath_name['results_content']][i].text_content(),
			}
			results.append(result)
		return results

	@property
	def links(self):
		reg = r"RU=(https?%3a%2f%2f[^/]+)/"
		tmp = self.framework.page_parse(self._pages).findall(reg)
		links = []
		for link in tmp:
			link = self.framework.urlib(link).unquote
			links.append(link)
		return links
	
	@property
	def dns(self):
		return self.framework.page_parse(self._pages).get_dns(self.q)

	@property
	def emails(self):
		return self.framework.page_parse(self._pages).get_emails(self.q)

	@property
	def docs(self):
		return self.framework.page_parse(self._pages).get_docs(self.q, self.links)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

class main:

	def __init__(self, q, limit=1):
		""" wikileaks.org search engine

			q		: Query for search
			limit	: Number of Pages

		"""

		self.framework = main.framework
		self.q = q
		self.agent = 'Mozilla/5.0 (X11; Linux x86_64; rv:86.0) Gecko/20100101 Firefox/86.0'
		self._pages = ''
		self.limit = limit
		self._links = []
		self._results = []
		self._links_with_data = []

	def run_crawl(self):
		from bs4 import BeautifulSoup as bs

		max_attempts = 0
		url = 'https://search.wikileaks.org/advanced'
		params = {'query': self.q, 'include_external_sources': True, 'page': 1}
		while True:
			self.framework.verbose(f"[WIKILEAKS] Searching in {params['page']} page ....", end='\r')
			try:
				req = self.framework.request(
					url=url,
					params=params,
					headers={'User-Agent': self.agent},
					allow_redirects=True)
			except Exception as e:
				self.framework.error(f"ConnectionError: {e}", 'util/wikileaks', 'run_crawl')
				max_attempts += 1
				if max_attempts == self.limit:
					self.framework.error('Wikileaks is missed!', 'util/wikileaks', 'run_crawl')
					return
			else:
				self._pages += req.text
				params['page'] += 1
				if params['page'] >= self.limit:
					break

		soup = bs(self._pages, 'html.parser')
		self._results = soup.find_all('div', class_='info')
		for result in self._results:
			self._links.append(result.find_all('a')[0]['href'])

	@property
	def pages(self):
		return self._pages
	
	@property
	def links(self):
		return self._links

	@property
	def links_with_data(self):
		for result in self._results:
			self._links_with_data.append({
				'title': result.find_all('a')[0].text,
				'link': result.find_all('a')[0]['href']
				})
		
		return self._links_with_data

	@property
	def dns(self):
		return self.framework.page_parse(self._pages).get_dns(self.q)

	@property
	def emails(self):
		return self.framework.page_parse(self._pages).get_emails(self.q)

	@property
	def docs(self):
		return self.framework.page_parse(self._pages).get_docs(self.q, self.links)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import requests
import json

class main :
	def __init__(self, q, limit=50, session_id=''):
		""" instagram.com search
			
			q 		  : The query for search
			limit	  : The number of details min 50 if exist
			session_id: Your Instagram session id
		"""
		
		self.q = q
		self.limit = int(limit/50) or 1
		self.session_id = session_id
		self.base_url = 'https://www.instagram.com/'
		self.headers = {'Content-Type': 'application/x-www-form-urlencoded'}
		if self.session_id :
			self.headers['Cookie'] = f"sessionid={self.session_id};"

		# request requrements
		self.data = {}
		self.posts_next_page = None
		self.followers_next_page = None
		self.following_next_page = None
		self.is_private = None

		# output_data
		self._userdata = {}
		self._followers = []
		self._following = []
		self._post = []

		self.session = requests.Session()

	def run_crawl(self) :
		self.framework.verbose('[INSTAGRAM] Extracting Data From API')
		# 1.1 account details find user on instagram 
		req = self.get_user_account_info()
		if req.status_code == 404 or self.data == {}:
			# user not found - show similar account names
			self.framework.error('Account not Found.', 'util/instagram', 'run_crawl')
			similar_users = self.get_similar_users()
			if similar_users :
				self.framework.heading('showing similar users', 0)

			for index, user in enumerate(similar_users, 1):
				self.framework.alert(f" {index}. user: {user['username']} --> {self.base_url + user['username']}")
			return

		if self.is_private or req.status_code != 200:
			# return if account is private or request fails
			if self.is_private: 
				self.framework.error('Account is Private.', 'util/instagram', 'run_crawl')
				if self.session_id == '':
					return

			else:
				self.framework.error(f"Request Fails, Url: {req.url}", 'util/instagram', 'run_crawl')
				return
		
		# 2. get post and follwers and following
		for _ in range(self.limit):
			if not self.get_user_posts():
				break
		
		for _ in range(self.limit):
			if not self.get_user_followers():
				break
		
		for _ in range(self.limit):
			if not self.get_user_following():
				break


	def get_user_account_info(self):
		"""extracting required user info form request"""
		req = self.session.get(url=f'{self.base_url}{self.q}/?__a=1', headers=self.headers)    
		try: 
			self.data = req.json()
		except:
			self.framework.error('Request Fail!! Too many tries', 'util/instagram', 'get_user_account_info')
			return
		if self.data:
			# id
			self._userdata['id'] = self.data['graphql']['user']['id']
			# username
			self._userdata['username'] = self.data['graphql']['user']['full_name']
			# bio
			self._userdata['bio'] = self.data['graphql']['user']['biography']
			# followers
			self._userdata['followers'] = self.data['graphql']['user']['edge_followed_by']['count']
			# following
			self._userdata['following'] = self.data['graphql']['user']['edge_follow']['count']
			# post 
			self._userdata['post_count'] = self.data['graphql']['user']['edge_owner_to_timeline_media']['count']
			# profile_pic
			self._userdata['profile_pic'] = self.data['graphql']['user']['profile_pic_url_hd']
			# is a private account
			self.is_private = self.data['graphql']['user']['is_private']
		return req

	def get_similar_users(self):
		"""return 20 similar users"""
		url = f"{self.base_url}web/search/topsearch/?query={self.q}"
		r = requests.get(url=url, headers=self.headers)
		# return first 20 similar names 
		return [i['user'] for i in r.json()['users']][:20]

	def get_user_posts(self):
		"""return boolean: if there is more data to extract"""
		url = f"{self.base_url}graphql/query/?"

		# set payload - id is target-id and have max limit is 50 at a request
		payload = {'id': self._userdata['id'], 'first': 50}
		if self.posts_next_page:
			payload['after'] = self.posts_next_page
		
		payload = json.dumps(payload)

		# encode the payload and add in params
		params = {'query_hash': '003056d32c2554def87228bc3fd9668a', 'variables': payload.encode()} 
		
		# geting response
		data = self.session.get(url, params=params, headers=self.headers).json()

		# extracting data from the response
		if data['status'] == 'ok':
			for node in data['data']['user']['edge_owner_to_timeline_media']['edges']:
				post = f"{self.base_url}p/{node['node']['shortcode']}"
				self._post.append(post)
				
			# have more posts 
			if data['data']['user']['edge_owner_to_timeline_media']['page_info']['has_next_page']:
				self.posts_next_page = data['data']['user']['edge_owner_to_timeline_media']['page_info']['end_cursor']
				return True
			else:
				self.posts_next_page = None
				return False
		else:
			self.framework.error('Request For collecting User POST is unsuccessful.', 'util/instagram', 'get_user_posts')
			return False

	def get_user_followers(self):
		"""return boolean: if there is more data to extract"""
		url = f"{self.base_url}graphql/query/?"

		# set payload - id is target-id and have max limit is 50 at a request
		payload = {'id': self._userdata['id'], 'include_reel': True, 'fetch_mutual': True, 'first': 50}
		if self.followers_next_page:
			payload['after'] = self.followers_next_page
		
		payload = json.dumps(payload)

		# encode the payload and add in params
		params = {'query_hash': 'c76146de99bb02f6415203be841dd25a', 'variables': payload.encode()}
		
		# geting response
		data = self.session.get(url, params=params, headers=self.headers).json()
		
		# extracting data from the response
		if data['status'] == 'ok':
			response_users = data['data']['user']['edge_followed_by']['edges']
			for node in response_users:
				user = {'id': node['node']['id'],
						'username': node['node']['username'],
						'profile_pic_url': node['node']['profile_pic_url']}
				self._followers.append(user)

			# have more followers 
			if data['data']['user']['edge_followed_by']['page_info']['has_next_page']:
				self.followers_next_page = data['data']['user']['edge_followed_by']['page_info']['end_cursor']
				return True
			else:
				self.followers_next_page = None
				return False

		else:
			self.framework.error('Request For collecting User Followers is unsuccessful.', 'util/instagram', 'get_user_followers')
			return False
	
	def get_user_following(self):
		"""return boolean: if there is more data to extract"""
		url = f"{self.base_url}graphql/query/?"

		# set payload - id is target-id and have max limit is 50 at a request
		payload = {'id': self._userdata['id'], 'include_reel': True, 'fetch_mutual': True, 'first': 50}
		if self.following_next_page:
			payload['after'] = self.following_next_page
		
		payload = json.dumps(payload)
		
		# encode the payload and add in params
		params = {'query_hash': '3dec7e2c57367ef3da3d987d89f9dbc8', 'variables': payload.encode()}
		
		# geting response
		data = self.session.get(url, params=params, headers=self.headers).json()
		
		# extracting data from the response
		if data['status'] == 'ok':
			response_users = data['data']['user']['edge_follow']['edges']
			for node in response_users:
				user = {'id': node['node']['id'],
						'username': node['node']['username'],
						'profile_pic_url': node['node']['profile_pic_url']}
				self._following.append(user)

			# have more followers 
			if data['data']['user']['edge_follow']['page_info']['has_next_page']:
				self.following_next_page = data['data']['user']['edge_follow']['page_info']['end_cursor']
				return True
			else:
				self.following_next_page  = None
				return False

		else:
			self.framework.error('[INSTAGRAM] Request For collecting User Following is unsuccessful.')
			return False


	@property
	def userdata(self):
		return self._userdata
	
	@property
	def followers(self):
		return self._followers
	
	@property
	def following(self):
		return self._following
	
	@property
	def post(self):
		return self._post



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from lxml import html


class main:
	def __init__(self, q):
		""" google.com image search engine
			q     : Query for search
		"""
		self.framework = main.framework
		self.q = q
		self.agent = 'Mozilla/5.0 (X11; Linux x86_64; rv:86.0) Gecko/20100101 Firefox/86.0'
		self.xpath_name_original = {
			'results': '//div[@class="isv-r PNCib MSM1fd BUooTd"]',
			'results_content': './/img',
			'results_title': './/h3',
			'results_a': './/a',
			'results_cite': './/div[@class="dmeZbb"]'
		}
		self.xpath_original = {
			self.xpath_name_original['results']: [
				self.xpath_name_original['results_content'],
				self.xpath_name_original['results_title'],
				self.xpath_name_original['results_a'],
				self.xpath_name_original['results_cite']
			]
		}
		self.url = 'https://www.google.com/search'
		self._pages = ''

	def run_crawl(self):
		payload = {'ie': 'utf-8', 'oe': 'utf-8', 'q': self.q, 'filter': '0', 'tbm': 'isch'}
		self.framework.verbose(f"[GOOGLE Images] Searching...", end='\r')
		try:
			req = self.framework.request(
				url=self.url,
				params=payload,
				headers={'user-agent': self.agent},
				allow_redirects=True)
		except Exception as e:
			self.framework.error(f"ConnectionError: {e}", 'util/google_images', 'run_crawl')
			self.framework.error('Google is missed!', 'util/google_images', 'run_crawl')
		else:
			if req.status_code in (503, 429):
				self.framework.error('Google CAPTCHA triggered.', 'util/google_images', 'run_crawl')
				return
			if req.status_code in (301, 302):
				redirect = req.headers['location']
				req = self.framework.request(url=redirect, allow_redirects=False)

			self._pages += req.text

	@property
	def results(self):
		tree = html.fromstring(self._pages)
		output = []
		results = []
		results = tree.xpath(self.xpath_name_original['results'])
		for i in results:
			a = i.xpath(self.xpath_name_original['results_content'])
			b = i.xpath(self.xpath_name_original['results_title'])
			c = i.xpath(self.xpath_name_original['results_a'])
			d = i.xpath(self.xpath_name_original['results_cite'])
			roler = {}
			if all([a,b,c,d]):
				if 'https://' not in a[0].values()[0]:
					continue
				roler['i'] = a[0].values()[0]
				roler['t'] = b[0].text_content()
				roler['a'] = c[1].get('href')
				roler['d'] = d[0].text_content()
			output.append(roler)
		return output

	@property
	def pages(self):
		return self._pages

	@property
	def links(self):
		links = [x['a'] for x in self.results]
		return links

	@property
	def dns(self):
		return self.framework.page_parse(self._pages).get_dns(self.q, self.links)

	@property
	def emails(self):
		return self.framework.page_parse(self._pages).get_emails(self.q)

	@property
	def docs(self):
		return self.framework.page_parse(self._pages).get_docs(self.q, self.links)
